#include "incl.h"
#include <math.h>

int CIPL_IdealPass(THEADER *, THEADER *,int, int ,int ,int, int ,int);
int CIPL_IdealReject(THEADER *, THEADER *,int, int ,int ,int, int ,int);
int CIPL_HSIIdealPass(THEADER *, THEADER *,int ,int);
int CIPL_HSIIdealReject(THEADER *, THEADER *,int ,int);
int CIPL_ButterworthPass(THEADER *, THEADER *,int, int ,int ,int, int ,int ,int);
int CIPL_ButterworthReject(THEADER *, THEADER *,int, int ,int ,int, int ,int ,int);
int CIPL_HSIButterworthPass(THEADER *, THEADER *,int, int ,int);
int CIPL_HSIButterworthReject(THEADER *, THEADER *,int, int ,int);

int CIPL_IdealPass(THEADER *it, THEADER *ot,int r, int g,int b,int br, int bg,int bb)
{
     int i, j, max;

     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				if(((r-br/2)<=it->iarr[i][j]) && (it->iarr[i][j]<=(r+br/2))
				  &&((g-bg/2)<=it->jarr[i][j]) && (it->jarr[i][j]<=(g+bg/2))
				  &&((b-bb/2)<=it->karr[i][j]) && (it->karr[i][j]<=(b+bb/2)))
				{
		    			ot->iarr[i][j] = it->iarr[i][j];
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
				else
				{	ot->iarr[i][j] = 0;
		    			ot->jarr[i][j] = 0;
		    			ot->karr[i][j] = 0;
				}
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_IdealReject(THEADER *it, THEADER *ot,int r, int g,int b,int br, int bg,int bb)
{
     int i, j, max;
     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				if(((r-br/2)<=it->iarr[i][j]) && (it->iarr[i][j]<=(r+br/2))
				  &&((g-bg/2)<=it->jarr[i][j]) && (it->jarr[i][j]<=(g+bg/2))
				  &&((b-bb/2)<=it->karr[i][j]) && (it->karr[i][j]<=(b+bb/2)))
				{
		    			ot->iarr[i][j] = 0;
		    			ot->jarr[i][j] = 0;
		    			ot->karr[i][j] = 0;
				}
				else
				{	ot->iarr[i][j] = it->iarr[i][j];
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_ButterworthPass(THEADER *it, THEADER *ot,int r, int g,int b,int br, int bg,int bb,int n)
{
     int i, j, max;
     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				
		    			ot->iarr[i][j] = (int)(it->iarr[i][j] * (1/(1+pow(((it->iarr[i][j]-r)/(br/2)),2*n))));
		    			ot->jarr[i][j] = (int)(it->jarr[i][j] * (1/(1+pow(((it->jarr[i][j]-g)/(bg/2)),2*n))));
		    			ot->karr[i][j] = (int)(it->karr[i][j] * (1/(1+pow(((it->karr[i][j]-b)/(bb/2)),2*n))));
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_ButterworthReject(THEADER *it, THEADER *ot,int r, int g,int b,int br, int bg,int bb,int n)
{
     int i, j, max;
     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				
		    			ot->iarr[i][j] = (int)(it->iarr[i][j] * (1 - (1/(1+pow(((it->iarr[i][j]-r)/(br/2)),2*n)))));
		    			ot->jarr[i][j] = (int)(it->jarr[i][j] * (1 - (1/(1+pow(((it->jarr[i][j]-g)/(bg/2)),2*n)))));
		    			ot->karr[i][j] = (int)(it->karr[i][j] * (1 - (1/(1+pow(((it->karr[i][j]-b)/(bb/2)),2*n)))));
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_HSIIdealPass(THEADER *it, THEADER *ot,int h, int bh)
{
     int i, j, max, diff;

     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				diff=abs(it->jarr[i][j]-h);
				if(diff > 1800)
					diff = 3600 -diff;
				if(diff < bh/2)
				{
		    			ot->iarr[i][j] = it->iarr[i][j];
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
				else
				{	ot->iarr[i][j] = 0;
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}
int CIPL_HSIIdealReject(THEADER *it, THEADER *ot,int h, int bh)
{
     int i, j, max, diff;

     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
				diff=abs(it->jarr[i][j]-h);
				if(diff > 1800)
					diff = 3600 -diff;
				if(diff < bh/2)
				{
		    			ot->iarr[i][j] = 0;
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
				else
				{	ot->iarr[i][j] = it->iarr[i][j];
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
				}
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_HSIButterworthPass(THEADER *it, THEADER *ot, int h, int bh, int n)
{
     int i, j, max, diff;
     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
					diff=it->jarr[i][j]-h;
					if(diff>=1800)
						diff=3600-diff;
		    			ot->iarr[i][j] = (int)(it->iarr[i][j] * (1/(1+pow((diff/(bh/2)),2*n))));
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

int CIPL_HSIButterworthReject(THEADER *it, THEADER *ot, int h, int bh, int n)
{
     int i, j, max, diff;
     ot->img.TYPE = it->img.TYPE;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }
	  
	  	for (i=0; i<max; i++) {
	       		for (j=0; j<it->img.COLS; j++) {
					diff=it->jarr[i][j]-h;
					if(diff>=1800)
						diff=3600-diff;
		    			ot->iarr[i][j] = (int)(it->iarr[i][j] * (1-(1/(1+pow((diff/(bh/2)),2*n)))));
		    			ot->jarr[i][j] = it->jarr[i][j];
		    			ot->karr[i][j] = it->karr[i][j];
	       		}
	  	}
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }

	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}






