#include "incl.h"

int CIPL_Differ(THEADER *it1,THEADER *it2,THEADER *ot);
int CIPL_SIMax(THEADER *it1,THEADER *it2,THEADER *ot);
int CIPL_SIAvg(THEADER *it1,THEADER *it2,THEADER *ot);
int CIPL_HFlip(THEADER *it,THEADER *ot);
int CIPL_Thresh(THEADER *it,THEADER *ot,int r,int g,int b,int flags);
int CIPL_Transpose(THEADER *it,THEADER *ot);
int CIPL_Bpslice(THEADER *it,THEADER *ot,int bit,int flags);

extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_ERRNO;
extern void HIPL_Error(char *a);

/****************************************************************
*	FUNCTION : CIPL_Differ					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Finds the difference between two image	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*      		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/

int CIPL_Differ(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS) || (it1->img.TYPE !=it2->img.TYPE))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	ot->img.TYPE = it1->img.TYPE;
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
			for(j=0;j<it1->img.COLS;j++)
			{
				ot->iarr[i][j]=abs(it1->iarr[i][j] - it2->iarr[i][j]);
				ot->jarr[i][j]=abs(it1->jarr[i][j] - it2->jarr[i][j]);
				ot->karr[i][j]=abs(it1->karr[i][j] - it2->karr[i][j]);
			}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);

}

/****************************************************************
*	FUNCTION : CIPL_SIMax					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Superimposes the two files by taking the 	*
*			Maximum of the two values		*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/


int CIPL_SIMax(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS)|| (it1->img.TYPE !=it2->img.TYPE))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	ot->img.TYPE = it1->img.TYPE;
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
		{
			for(j=0;j<it1->img.COLS;j++)
			{
				if(it1->iarr[i][j]>=it2->iarr[i][j])
					ot->iarr[i][j]=it1->iarr[i][j];
				else
					ot->iarr[i][j]=it2->iarr[i][j];
				
				if(it1->jarr[i][j]>=it2->jarr[i][j])
					ot->jarr[i][j]=it1->jarr[i][j];
				else
					ot->jarr[i][j]=it2->jarr[i][j];
				
				if(it1->karr[i][j]>=it2->karr[i][j])
					ot->karr[i][j]=it1->karr[i][j];
				else
					ot->karr[i][j]=it2->karr[i][j];
			}
		}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);

}

/****************************************************************
*	FUNCTION : CIPL_SIAvg					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Superimposes the two files by taking the 	*
*		Average of the two values			*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/

int CIPL_SIAvg(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS)|| (it1->img.TYPE !=it2->img.TYPE))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	ot->img.TYPE = it1->img.TYPE;
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
			for(j=0;j<it1->img.COLS;j++)
			{
					ot->iarr[i][j]=(int)((it1->iarr[i][j] + it2->iarr[i][j])/2);
					ot->jarr[i][j]=(int)((it1->jarr[i][j] + it2->jarr[i][j])/2);
					ot->karr[i][j]=(int)((it1->karr[i][j] + it2->karr[i][j])/2);
			}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);
}

/*************************************************************************/

int CIPL_HFlip(THEADER *it,THEADER *ot)
{
	int i,max,j,cols;
	ot->img.TYPE = it->img.TYPE;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		cols = it->img.COLS;
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++)
			{
				ot->iarr[i][cols-j-1]=it->iarr[i][j];
				ot->jarr[i][cols-j-1]=it->jarr[i][j];
				ot->karr[i][cols-j-1]=it->karr[i][j];
			}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Putpart in Horizontal Flip");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);
}

/*************************************************************************/

int CIPL_Thresh(THEADER *it,THEADER *ot,int r,int g,int b,int flags)
{
	int i,max,j;
	ot->img.TYPE = it->img.TYPE;
	
	if ((flags & 1) != 1)
		r=0;
	if ((flags & 2) != 2)
		g=0;
	if ((flags & 4) != 4)
		b=0;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++)
			{
				if(it->iarr[i][j] >= r && it->jarr[i][j] >= g && it->karr[i][j] >= b)
				{
					ot->iarr[i][j]=it->iarr[i][j];
					ot->jarr[i][j]=it->jarr[i][j];
					ot->karr[i][j]=it->karr[i][j];
				}
				else
				{
					ot->iarr[i][j]=0;
					ot->jarr[i][j]=0;
					ot->karr[i][j]=0;
				}
			}
		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}


/*************************************************************************/


int CIPL_Transpose(THEADER *it,THEADER *ot)
{
	int i,max1,x,j,c,p,part,r,T_Parts,max2;
	ot->img.TYPE = it->img.TYPE;
	if(it->img.ROWS!=it->img.COLS)
		{
			HIPL_ERRNO=-29;
			HIPL_Error("Error in Operation");
			return -1;
		}
	T_Parts=it->img.ROWS/it->P.psize;
	part=0;
	r=0;
	do
	{
		c=0;
		do
		{
		max1=HIPL_Getpart(it);
		if(max1<0)
		{
			HIPL_Error("Error in Getpart");
			return(max1);
		}
		if(part==T_Parts)
		{
		x=it->img.COLS%it->P.psize;
		r=(it->img.COLS/it->P.psize)*it->P.psize;
		}
		else x=it->P.psize;
		for(i=0;i<max1;i++)
			for(j=0;j<x;j++){
				
				ot->iarr[j][i+c]=it->iarr[i][j+r];
				ot->jarr[j][i+c]=it->jarr[i][j+r];
				ot->karr[j][i+c]=it->karr[i][j+r];
			}
				
		c+=it->P.psize;
		}while(!(it->P.nleft==0));
		HIPL_Refresh(it,0);
		max2=HIPL_Putpart(ot);
		if(max2<0)
		{
			HIPL_Error("Error in Putpart");
			return(max2);
		}
		r+=it->P.psize;
		part++;
		p+=max1;
	}while(ot->P.nleft!=0);
	return(0);
}

int CIPL_Bpslice(THEADER *it,THEADER *ot,int bit,int flags){
	int i,max,j,l;
	if ((bit>7)||(bit<0)) {
		HIPL_ERRNO=-27;
		HIPL_Error("Error in Operation");
		return -1;
	}
	if (flags !=1 && flags !=2 && flags !=4) {
		HIPL_ERRNO=-32;
		HIPL_Error("Error in Operation");
		return -1;
	}
	ot->img.TYPE = PGM;
	l=1;
	for(i=0;i<bit;i++) l<<=1;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++) {
			if(flags == 1)
			{
				ot->iarr[i][j]=it->iarr[i][j]&l;
				if(ot->iarr[i][j]) 
					ot->iarr[i][j]=255;
			}
			else if(flags == 2)
			{
				ot->iarr[i][j]=it->jarr[i][j]&l;
				if(ot->iarr[i][j]) 
					ot->iarr[i][j]=255;
			}
			else if(flags == 4)
			{
				ot->iarr[i][j]=it->karr[i][j]&l;
				if(ot->iarr[i][j]) 
					ot->iarr[i][j]=255;
			}
		}
		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);
	return(0);
}
