#include "incl.h"

int CIPL_Gaussavg(THEADER *it,THEADER *ot,int sigma,int flags);
int CIPL_Avg(THEADER *T1,THEADER *T2,float *G,int sigma,int flags);
int CIPL_CompGMask(float *GMask,int sigma);

extern int HIPL_Initialise(THEADER *t,char *name,short i,short j,DATA *ID);
extern int HIPL_AllocateMem(THEADER *);
extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_ERRNO;
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_FreeMem(THEADER *T);
extern int HIPL_Refresh(THEADER *T,int i);
extern void HIPL_Error(char *a);



int CIPL_Gaussavg(THEADER *it,THEADER *ot,int sigma, int flags)
{
	THEADER *gt;
	float *GMask;
	int i,j,k,msk;
	DATA *GID;
	gt=(THEADER *)malloc(sizeof(THEADER));
	if(gt==(THEADER *)0)
	{
		HIPL_ERRNO=-2;
		HIPL_Error("INPUT HEADER");
		return(-1);
	}
	GID=(DATA*)malloc(sizeof(DATA));
	GID->ROWS=it->img.ROWS;
	GID->COLS=it->img.COLS;
	GID->type=it->img.TYPE;
	GID->ver=it->ver;

	ot->img.TYPE = it->img.TYPE;
	
	i=HIPL_Initialise(gt,NULL,1,0,GID);
	
	if(i<0)
	{
		HIPL_Error("Initialising output");
		return(-1);
	}
	
	
	i=HIPL_AllocateMem(gt);
	if(i<0)
	{
		HIPL_Error("Initialising output");
		return(-1);
	}
	msk=(6*sigma)+1;
	if(it->P.psize<=msk)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Copy operation");
		return(-1);
	}
	

	GMask=(float *)malloc(msk*sizeof(float));
	if(GMask==(float *)0)
	{
		HIPL_ERRNO=-9;
		HIPL_Error("Gaussian Average");
		return(-1);
	}

	i=CIPL_CompGMask(GMask,sigma);
	
	i=CIPL_Avg(it,gt,GMask,sigma,flags);
	gt->MROWS=0;
	gt->MCOLS=0;
	i=HIPL_Refresh(gt,0);
	
	do
	{
		i=HIPL_Getpart(gt);
	
		if(i<0)
		{
			HIPL_Error("Getpart in copy");
			return(i);
		}


		for(j=0;j<i;j++)
		{	for(k=0;k<gt->img.COLS;k++){
				
					ot->iarr[j][k]=gt->iarr[j][k];
				
					ot->jarr[j][k]=gt->jarr[j][k];
			
					ot->karr[j][k]=gt->karr[j][k];
			}
		}

		i=HIPL_Putpart(ot);

		if(i<0)
		{
			HIPL_Error("Putpart in copy");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);


	i=HIPL_Closeall(gt);
	if(i!=0)
	{
		HIPL_Error("Gaussian average - output header1");
		return(-1);
	}
	i=HIPL_FreeMem(gt);
	if(i!=0)
	{
		HIPL_Error("Gaussian average - output header1");
		return(-1);
	}
	free(GMask);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}

int CIPL_CompGMask(float *GMask,int sigma)
{
	float gmax,a,b,c,d,e,pi;
	int i,j;
	j=(int)sigma;
	pi=(float)M_PI;
	a=(float)sigma;
	b=(float)sqrt(2*pi);
	c=b*a;
	d=2*a*a;
	gmax=(float)1.0/c;

	for(i=1;i<=(3*j);i++)
	{
		e=(float)((-1)*i*i);
		GMask[(3*j)-i]=(float)gmax*(float)exp(e/d);
		GMask[(3*j)+i]=GMask[(3*j)-i];
	}

	GMask[3*j]=gmax;
	return(0);

}


int CIPL_Avg(THEADER *it,THEADER *gt,float *GMask,int sigma,int flags)

{

	int i,j,fus,hs,max,k,*temp4,max1;
	float **temp1,**temp2,**temp3,t1,temp;
	
	

	fus=(6*sigma)+1;
	hs=fus/2;
	it->MROWS=gt->MROWS=fus;
	it->MORGX=gt->MORGX=hs;
	
	if((it->img.TYPE == HSV) && ((flags & 2) ==2 ))
	{	temp4=(int*)malloc(sizeof(int)*it->MROWS);
		
	}
	
	temp1=(float **)malloc(it->P.psize*sizeof(float *));
	temp2=(float **)malloc(it->P.psize*sizeof(float *));
	temp3=(float **)malloc(it->P.psize*sizeof(float *));
	if((temp1==(float **)0)||(temp2==(float **)0)||(temp3==(float **)0))
	{
		HIPL_ERRNO=-5;
		HIPL_Error("Temporary array in Gaussian Average");
		return(-1);
	}

	for(i=0;i<it->P.psize;i++)
	{
		temp1[i]=(float *)malloc(it->img.COLS * sizeof(float));
		temp2[i]=(float *)malloc(it->img.COLS * sizeof(float));
		temp3[i]=(float *)malloc(it->img.COLS * sizeof(float));
		if((temp1[i]==(float *)0)||(temp2[i]==(float *)0)||(temp3[i]==(float *)0))
		{
			HIPL_ERRNO=-5;
			HIPL_Error("Temporary array in Gaussian Average");
			return(-1);
		}
	}	
	
	
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Getpart in Gaussian Average");
			return(i);
		}

		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				if((j<hs)||(j>=(it->img.COLS-hs)))
				{
					temp1[i][j]=(float) it->iarr[i][j];
					temp2[i][j]=(float) it->jarr[i][j];
					temp3[i][j]=(float) it->karr[i][j];
				}
				else
				{	if((flags & 1) == 1)
					{
						t1=0;
						for(k=-hs;k<=hs;k++)
						{
							temp=GMask[k+hs]*((float)it->iarr[i][j+k]); 
							t1=t1+temp;
						}
						temp1[i][j]=t1;
					}
					else
						temp1[i][j]=it->iarr[i][j];
					if((flags & 2) == 2)
					{	if(it->img.TYPE == HSV)
						{	
							max1=-1;
							for(k=-hs;k<=hs;k++)
							{
								if(max1<it->jarr[i][j+k])
									max1 = it->jarr[i][j+k];
															
							}
							
							for(k=-hs;k<=hs;k++)
							{
								if(it->jarr[i][j+k]<(max1 - 1800))
									temp4[hs+k]= it->jarr[i][j+k] + 3600;
								else
									temp4[hs+k] = it->jarr[i][j+k];
									
							}
					
						}
						t1=0;
						for(k=-hs;k<=hs;k++)
						{	if(it->img.TYPE == HSV)
								temp=GMask[k+hs]*((float)temp4[hs+k]); 
							else
								temp=GMask[k+hs]*((float)it->jarr[i][j+k]); 
							t1=t1+temp;
						}
						if(t1>3600)
							temp2[i][j]=t1-3600;
						else
							temp2[i][j]=t1;
					}
					else
						temp2[i][j]=it->jarr[i][j];
					if((flags & 4) == 4)
					{
						t1=0;
						for(k=-hs;k<=hs;k++)
						{
							temp=GMask[k+hs]*((float)it->karr[i][j+k]); 
							t1=t1+temp;
						}
						temp3[i][j]=t1;
					}
					else
						temp3[i][j]=it->karr[i][j];
					
				}
				
			}
		}


		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				if((i<hs)||(i>=(max-hs)))
				{	
					gt->iarr[i][j]=(int)temp1[i][j];
					gt->jarr[i][j]=(int)temp2[i][j];
					gt->karr[i][j]=(int)temp3[i][j];
					
				}
				else
				{	if((flags & 1)==1)
					{
						t1=0;
						for(k=-hs;k<=hs;k++)
						{
							temp=GMask[k+hs]*(temp1[i+k][j]);
							t1=t1+temp;
						}
						gt->iarr[i][j]=(int)t1;
					}
					else
						gt->iarr[i][j]=temp1[i][j];
					if((flags & 2)==2)
					{	if(it->img.TYPE == HSV)
						{	
							max1=-1;
							for(k=-hs;k<=hs;k++)
							{
								if(max1<temp2[i+k][j])
									max1 = temp2[i+k][j];
															
							}
							
							for(k=-hs;k<=hs;k++)
							{
								if(temp2[i+k][j]<(max1 - 1800))
									temp4[hs+k]= temp2[i+k][j] + 3600;
								else
									temp4[hs+k] = temp2[i+k][j];
									
							}
					
						}
						t1=0;
						for(k=-hs;k<=hs;k++)
						{	if(it->img.TYPE == HSV)
								temp=GMask[k+hs]*(temp4[hs+k]);
							else
								temp=GMask[k+hs]*(temp2[i+k][j]);
							t1=t1+temp;
						}
						gt->jarr[i][j]=(int)t1;
						if(gt->jarr[i][j]>3600)
							gt->jarr[i][j]-=3600;
					}
					else
						gt->jarr[i][j]=temp2[i][j];
					if((flags & 4)==4)
					{
						t1=0;
						for(k=-hs;k<=hs;k++)
						{
							temp=GMask[k+hs]*(temp3[i+k][j]);
							t1=t1+temp;
						}
						gt->karr[i][j]=(int)t1;
					}
					else
						gt->karr[i][j]=temp3[i][j];
				}
				
			}
		}


		i=HIPL_Putpart(gt);
		if(i<0)
		{
			HIPL_Error("Putpart in Gaussian Average");
			return(i);
		}


		if(gt->P.nleft==0)
			i=-1;
	}while(i>=0);

	for(i=0;i<it->P.psize;i++)
	{
		free(temp1[i]);
		free(temp2[i]);
		free(temp3[i]);
	}
	free(temp1);
	free(temp2);
	free(temp3);
	if((it->img.TYPE == HSV)&&((flags & 2) == 2))
		free(temp4);
	return(0);

}

