#include "incl.h"

extern int HIPL_Initialise(THEADER *t,char *name,short i,short j,DATA *ID);
extern int HIPL_AllocateMem(THEADER *t);
extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_FreeMem(THEADER *t);
extern int HIPL_Refresh(THEADER *T,int flag);
extern void HIPL_Error(char *msg);
extern int HIPL_ERRNO;

int CIPL_Str(THEADER *it,THEADER *ot,long int *hist,int gmax,int gmin,int flags);
int CIPL_Equalise(THEADER *it,THEADER *ot,long int *hist, int flags);
int CIPL_FindHistogram(THEADER *it,DATA *ID,int levels,int color);

/****************************************************************
*	FUNCTION : CIPL_Str					*
*								*
*	INPUT	 : Input header,Output header  Maximum and	*
*		    Minimum Saturation values in the image	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Stretches the histogram to the user specifed	*
*			Saturation levels			*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
*								*
****************************************************************/

int CIPL_Str(THEADER *it,THEADER *ot,long int *hist,int gmax,int gmin,int flags)
{
	int i,j,k,tmax,tmin,hist_max;
	float val,val1,val2,value;
	ot->img.TYPE = it->img.TYPE;
	if(flags != 1 && flags != 2 && flags !=4)
		return 1;
	if(it->img.TYPE == 50 )
	{
		if(flags == 2)
			hist_max = 360;
		else if(flags == 4)
			hist_max = 100;
		else 
			hist_max = 256;
	}
	else
		hist_max = 256;
	
	for(i=0;i<hist_max;i++)
	{
		if(hist[i]!=0)
		{
			tmin=i;
			break;
		}
	}
	for(i=hist_max-1;i>=0;i--)
	{
		if(hist[i]!=0)
		{
			tmax=i;
			break;
		}
	}
	if(tmax<=tmin)
		return(0);
	if(gmax>hist_max-1)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	if(gmin<0)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	if(gmax<=gmin)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	val1=(float)(gmax-gmin);
	val2=(float)(tmax-tmin);
	value=val1/val2;
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Histogram Stretch");
			return(i);
		}
		for(j=0;j<i;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				if( flags == 1)
				{
					val1=(float)(it->iarr[j][k]-tmin);
					val2=value*val1;
					val=val2+((float)gmin);
					ot->iarr[j][k]=(int)val;
					ot->karr[j][k]=it->karr[j][k];
					ot->jarr[j][k]=it->jarr[j][k];
					if(ot->iarr[j][k]>hist_max)
						ot->iarr[j][k]=hist_max;
					if(ot->iarr[j][k]<0)
						ot->iarr[j][k]=0;
				}
				else if( flags == 2)
				{
					val1=(float)(it->jarr[j][k]-tmin);
					val2=value*val1;
					val=val2+((float)gmin);
					ot->jarr[j][k]=(int)val;
					ot->iarr[j][k]=it->iarr[j][k];
					ot->karr[j][k]=it->karr[j][k];
					if(ot->jarr[j][k]>hist_max)
						ot->jarr[j][k]=hist_max;
					if(ot->jarr[j][k]<0)
						ot->jarr[j][k]=0;
				}
				else if( flags == 4)
				{
					val1=(float)(it->karr[j][k]-tmin);
					val2=value*val1;
					val=val2+((float)gmin);
					ot->karr[j][k]=(int)val;
					ot->iarr[j][k]=it->iarr[j][k];
					ot->jarr[j][k]=it->jarr[j][k];
					if(ot->karr[j][k]>hist_max)
						ot->karr[j][k]=hist_max;
					if(ot->karr[j][k]<0)
						ot->karr[j][k]=0;
				}
			}
		}
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Histogram Stretch");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}

/****************************************************************
*	FUNCTION : CIPL_Equalise				*
*								*
*	INPUT	 :  Input header,Output header     		*
*		    						*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Equlises the histogram to the user specifed	*
*			Saturation levels			*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
****************************************************************/

int CIPL_Equalise( THEADER *it,THEADER *ot,long int *hist,int flags)
{
	int i,j,k,m,max,hist_max;
	float cont,t1,t2;
	float area=0,*sum_hist;
	
	ot->img.TYPE = it->img.TYPE;
	if(flags != 1 && flags != 2 && flags !=4)
		return -1;
	if(it->img.TYPE == 50 )
	{
		if(flags == 2)
			hist_max = 360;
		else if(flags == 4)
			hist_max = 100;
		else 
			hist_max = 256;
	}
	else
		hist_max = 256;
	sum_hist = (float * ) malloc(sizeof( float )*hist_max);
		
	
	for(i=0;i<hist_max;i++)
		sum_hist[i]=0;
	for(i=0;i<hist_max;i++)
	{	area=area+(float)hist[i];
		sum_hist[i]=area;
	}
	area=((float)it->img.ROWS*(float)it->img.COLS);
	t2=(float)hist_max;
	t1=(float)area;
	if(t1==0)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	cont=t2/t1;
	do
	{
		max=0;
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Histogram Equalise");
			return(i);
		}
		for(j=0;j<max;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				if(flags == 1)
				{
					m=it->iarr[j][k];
					t1=(float)sum_hist[m];
					t2=t1*cont;
					ot->iarr[j][k]=(int)t2;
					ot->karr[j][k]=it->karr[j][k];
					ot->jarr[j][k]=it->jarr[j][k];
					if(ot->iarr[j][k]>=hist_max)
						ot->iarr[j][k]=hist_max-1;
					if(ot->iarr[j][k]<0)
						ot->iarr[j][k]=0;
				}
				else if(flags == 2)
				{
					m=it->jarr[j][k];
					t1=(float)sum_hist[m];
					t2=t1*cont;
					ot->jarr[j][k]=(int)t2;
					ot->iarr[j][k]=it->iarr[j][k];
					ot->karr[j][k]=it->karr[j][k];
					if(ot->jarr[j][k]>=hist_max)
						ot->jarr[j][k]=hist_max-1;
					if(ot->jarr[j][k]<0)
						ot->jarr[j][k]=0;
				}
				else if(flags == 4)
				{
					m=it->karr[j][k];
					t1=(float)sum_hist[m];
					t2=t1*cont;
					ot->karr[j][k]=(int)t2;
					ot->iarr[j][k]=it->iarr[j][k];
					ot->jarr[j][k]=it->jarr[j][k];
					if(ot->karr[j][k]>=hist_max)
						ot->karr[j][k]=hist_max-1;
					if(ot->karr[j][k]<0)
						ot->karr[j][k]=0;
				}
			}
		}
		max=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Histogram Equalise");
			return(i);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);
	free(sum_hist);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}
/****************************************************************
*	FUNCTION : CIPL_FindHistogram			*
*								*
*	INPUT	 : Input Header, HIstogram array		*
*								*
*	OUTPUT	 :  0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Calculates the average gray level and the 	*
*		            standard deviation of gray levels	*
*								*
*  EXTERNAL CALLS :  1)HIPL_Getpart				*
*		  						*
*								*
*		     2)HIPL_Refresh				*
*		        					*
*								*
****************************************************************/

int CIPL_FindHistogram(THEADER *it,DATA *ID,int levels,int flags)
{
	long int x;
	int i,j,k;
	
	ID->hist=(long int*)malloc(levels*sizeof(long int));
	for(i=0;i<levels;i++)
		ID->hist[i]=0;
	
	do
	{
		x=0;
		i=HIPL_Getpart(it);
		if(i<0)
			return(i);
			
		for(j=0;j<i;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{	if(flags==1)
					x=it->iarr[j][k];
				else if(flags==2)
					x=it->jarr[j][k];
				else if (flags == 4)
					x=it->karr[j][k];
				if(x>(levels-1))
				{	if(it->img.TYPE == HSV && flags == 2)
						x=levels-360;
					else
						x=levels-1;
				}
				if(x<0)
					x=0;
				ID->hist[x]++;
			}
		}
	}while(i>0);
	
	HIPL_Refresh(it,0);
	return(0);
}




int CIPL_Lstretch(THEADER *it,THEADER *ot,int flags)
{
	int i,j,k,x,*L,temp,max=-1,max_val;
	double base;
	DATA *ID;
	
	if(flags !=1 && flags!=2 && flags!=4)
		return -1;
	ot->img.TYPE=it->img.TYPE;
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=it->img.ROWS;
	ID->COLS=it->img.COLS;
	ID->ver=it->ver;
	ID->type=it->img.TYPE;
	
	if((it->img.TYPE == HSV) && (flags ==2))
		max_val=3600;
	else if((it->img.TYPE == HSV) && (flags ==4))
		max_val=1000;
	else
		max_val=256;
	L=(int *)malloc(max_val*sizeof(int));
	
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
			return(i);
		for(j=0;j<i;j++)
		{	for(k=0;k<it->img.COLS;k++)
			{	if(flags == 1)
				{	if(it->iarr[j][k]>max)
						max=it->iarr[j][k];
				}
				else if(flags == 2)
				{	if(it->jarr[j][k]>max)
						max=it->jarr[j][k];
				}
				else if(flags == 4)
				{	if(it->karr[j][k]>max)
						max=it->karr[j][k];
				}
			}	
		}
	}while(i>0);
	
	HIPL_Refresh(it,0);
	if( max > 1)
		base=pow(10,log10((double)(max))/(double)max_val);
	
	switch(max) {
		case 0 : 
			L[0] = 255;
			break;
		case 1 :
			L[1] = 255;
			L[0] = 0;
			break;
		default :
			for(i=0;i<max_val;i++) {
				if(i==0) {
					temp = 0;
				} else {
					temp=(int)(log10((double)(i))/log10(base));
				}
				if(temp > (max_val-1))
					temp = max_val-1;
				L[i]=temp;
			}
			break;
		}
	HIPL_Refresh(it,0);	
	HIPL_Refresh(ot,0);
	
	do
	{
		x=0;
		i=HIPL_Getpart(it);
		if(i<0)
			return(i);
		for(j=0;j<i;j++)
			for(k=0;k<it->img.COLS;k++)
			{	if(flags == 1)
				{	ot->iarr[j][k]=L[it->iarr[j][k]];
					ot->jarr[j][k]=it->jarr[j][k];
					ot->karr[j][k]=it->karr[j][k];
				}
				else if(flags == 2)
				{	ot->jarr[j][k]=L[it->jarr[j][k]];
					ot->iarr[j][k]=it->iarr[j][k];
					ot->karr[j][k]=it->karr[j][k];
				}
				else if(flags == 4)
				{	ot->karr[j][k]=L[it->karr[j][k]];
					ot->jarr[j][k]=it->jarr[j][k];
					ot->iarr[j][k]=it->iarr[j][k];
				}
			}
		i=HIPL_Putpart(ot);
		if(i<0)
			return(i);
	}while(ot->P.nleft);
	
	free(ID);
	HIPL_Refresh(it,0);	
	HIPL_Refresh(ot,0);
	return(0);
}
