#include "incl.h"
int HIPL_Differ(THEADER *it1,THEADER *it2,THEADER *ot);
int HIPL_SIMax(THEADER *it1,THEADER *it2,THEADER *ot);
int HIPL_SIAvg(THEADER *it1,THEADER *it2,THEADER *ot);
int HIPL_HFlip(THEADER *it,THEADER *ot);
int HIPL_Refresh(THEADER *,short);
int HIPL_Range(THEADER *it,THEADER *ot,int lo,int hi);
extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_ERRNO;
extern void HIPL_Error(char *a);

/****************************************************************
*	FUNCTION : HIPL_Differ					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Finds the difference between two image	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*      		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/

int HIPL_Differ(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
			for(j=0;j<it1->img.COLS;j++)
				ot->iarr[i][j]=abs(it1->iarr[i][j] - it2->iarr[i][j]);

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);

}

/****************************************************************
*	FUNCTION : HIPL_SIMax					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Superimposes the two files by taking the 	*
*			Maximum of the two values		*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/


int HIPL_SIMax(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
		{
			for(j=0;j<it1->img.COLS;j++)
			{
				if(it1->iarr[i][j]>=it2->iarr[i][j])
					ot->iarr[i][j]=it1->iarr[i][j];
				else
					ot->iarr[i][j]=it2->iarr[i][j];
			}
		}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);

}

/****************************************************************
*	FUNCTION : HIPL_SIAvg					*
*								*
*	INPUT	 : source and target file headers		*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Superimposes the two files by taking the 	*
*		Average of the two values			*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
****************************************************************/

int HIPL_SIAvg(THEADER *it1,THEADER *it2,THEADER *ot){
	int i,j,max;
	if((it1->img.ROWS!=it2->img.ROWS)||(it1->img.COLS!=it2->img.COLS))
	{
		HIPL_ERRNO=-26;
		HIPL_Error("Error in Operation");
		return -1;
	}	
	
	do
	{
		max=HIPL_Getpart(it1);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		max=HIPL_Getpart(it2);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}

		for(i=0;i<max;i++)
			for(j=0;j<it1->img.COLS;j++)
					ot->iarr[i][j]=(int)((it1->iarr[i][j] + it2->iarr[i][j])/2);

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);
}

/*************************************************************************/

int HIPL_HFlip(THEADER *it,THEADER *ot){
int i,max,j,cols;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		cols = it->img.COLS;
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++)
					ot->iarr[i][cols-j-1]=it->iarr[i][j];

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Putpart in Horizontal Flip");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);
}

/*************************************************************************/

int HIPL_Thresh(THEADER *it,THEADER *ot,int grey){
int i,max,j;
	if((grey>255)||(grey<0)) {
		HIPL_ERRNO=-28;
		HIPL_Error("Error in Operation");
		return -1;
		}
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++)
				if(it->iarr[i][j] >= grey) 
					ot->iarr[i][j]=255;
				else
					ot->iarr[i][j]=0;
		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);

	return(0);
}

/*************************************************************************/

int HIPL_Range(THEADER *it,THEADER *ot,int lo,int hi){
int i,max,j;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++) {
				if((it->iarr[i][j]<=lo)||(it->iarr[i][j]>=hi)) ot->iarr[i][j]=0;
			     else 
				ot->iarr[i][j]=it->iarr[i][j];
					}
		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
	}while(ot->P.nleft);
	return(0);
}

/*************************************************************************/

int HIPL_Bpslice(THEADER *it,THEADER *ot,int bit){
	int i,max,j,l;
	if ((bit>7)||(bit<0)) {
		HIPL_ERRNO=-27;
		HIPL_Error("Error in Operation");
		return -1;
	}
	ot->img.TYPE = PGM;
	l=1;
	for(i=0;i<bit;i++) l<<=1;
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++) {
			ot->iarr[i][j]=it->iarr[i][j]&l;
			if(ot->iarr[i][j]) ot->iarr[i][j]=255;
		}
		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);
	return(0);
}

/*************************************************************************/

int HIPL_Transpose(THEADER *it,THEADER *ot){
int i,max1,x,j,c,p,part,r,T_Parts,max2;
	if(it->img.ROWS!=it->img.COLS)
		{
			HIPL_ERRNO=-29;
			HIPL_Error("Error in Operation");
			return -1;
		}
	T_Parts=it->img.ROWS/it->P.psize;
	part=0;
	r=0;
	do
	{
		c=0;
		do
		{
		max1=HIPL_Getpart(it);
		if(max1<0)
		{
			HIPL_Error("Error in Getpart");
			return(max1);
		}
		if(part==T_Parts)
		{
		x=it->img.COLS%it->P.psize;
		r=(it->img.COLS/it->P.psize)*it->P.psize;
		}
		else x=it->P.psize;
		for(i=0;i<max1;i++)
			for(j=0;j<x;j++){
				if(it->arrtype==0)
				ot->iarr[j][i+c]=it->iarr[i][j+r];
				else
				ot->farr[j][i+c]=it->farr[i][j+r];
				}
		c+=it->P.psize;
		}while(!(it->P.nleft==0));
		HIPL_Refresh(it,0);
		max2=HIPL_Putpart(ot);
		if(max2<0)
		{
			HIPL_Error("Error in Putpart");
			return(max2);
		}
		r+=it->P.psize;
		part++;
		p+=max1;
	}while(ot->P.nleft!=0);
	return(0);
}

int HIPL_ImageToArray(THEADER *img, int **arr, int xmin, int ymin,
		      int xmax, int ymax)
{
     int i, j;

     for (i=0; i<(xmax-xmin); i++)
	  for (j=0; j<(ymax-ymin); j++)
	       arr[i][j] = img->iarr[xmin+i][ymin+j];

     return(0);
}
