#include "incl.h"
extern int HIPL_ERRNO;
extern  int HIPL_Initialise(THEADER *T,char *name,short io,short arr_type,DATA *ID);
extern int HIPL_AllocateMem(THEADER *T);
extern  int HIPL_Getpart(THEADER *T);
extern  int HIPL_Putpart(THEADER *T);
extern  void HIPL_Error(char *msg);
extern  int HIPL_Closeall(THEADER *T);
extern int HIPL_FreeMem(THEADER *T);
extern  int HIPL_Refresh(THEADER *T,int flg);

/****************************************************************
*	FUNCTION : HIPL_BiDilate				*
*								*
*	INPUT	 : source and target file names			*
*		    The mask,mask size and the dividing factor	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Applies the mask on the source file and	*
*			places the result in the target file	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*			File : HIPL_Coreio.c			*
*								*
****************************************************************/

int HIPL_BiDilate(THEADER *it,THEADER *ot,int **Mask)
{
	int i,j,max,m,n,l,k;
	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Error in Operation");
		return(-1);
	}
	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++)
			ot->iarr[i][j]=0;
		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				m=-it->MORGY;
				n=-it->MORGX;
				if((i<it->MORGX)||((max - i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
						ot->iarr[i][j]=it->iarr[i][j];
				else
				{
				if(it->iarr[i][j]==255)
					for(k=0;k<it->MROWS;k++){
						for(l=0;l<it->MCOLS;l++){
						if(Mask[k][l]==255)
						ot->iarr[i+n][j+m]=255;
						m++;
						}
						m=-it->MORGY;
						n++;
					}
				}
			}
		}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
	}while(ot->P.nleft);
	
	return(0);

}

/****************************************************************
*	FUNCTION : HIPL_BiErode					*
*								*
*	INPUT	 : Input and  the Output headers		*
*		    the mask,number of rows to be processed	*
*		     and the reciprocal of the factor		*
*								*	
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Moves the mask on the array in the input     *
*		   header and places the result in the output	*
*		   array.This function is called repeatedly by	*
*		    HIPL_Maskimg function ie for every Getpart	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*			File : HIPL_Coreio.c			*
*								*
****************************************************************/

int HIPL_BiErode(THEADER *it,THEADER *ot,int **Mask)
{
	int i,j,n,m,k,l,max;
	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Error in Operation");
		return(-1);
	}

	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
		for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++)
			ot->iarr[i][j]=0;
		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				m=-it->MORGY;
				n=-it->MORGX;
				if((i<it->MORGX)||((max - i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
						ot->iarr[i][j]=it->iarr[i][j];
				else
				{
				if(it->iarr[i][j]==255)
				{
				for(k=0;k<it->MROWS;k++){
				for(l=0;l<it->MCOLS;l++){
				if(Mask[k][l]!=it->iarr[i+n][j+m])
				break;
				m++;
				}
				m=-it->MORGY;
				n++;
				if(l!=it->MCOLS)
				break;
				}
				if(k==it->MROWS)
				ot->iarr[i][j]=255;
				}
				}
			}
		}

		max=HIPL_Putpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Putpart");
			return(max);
		}
	}while(ot->P.nleft);
	
	return(0);

}

/****************************************************************
*	FUNCTION : HIPL_BiOpen					*
*								*
*	INPUT	 : source and target file names and mask size	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Applies the median filter on the source file * 
*			and places the result in the target file*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*		     6)HIPL_Closeall				*
*			File : HIPL_Coreio.c			*
*								*
****************************************************************/

int HIPL_BiOpen(THEADER *it,THEADER *ot,int **Mask)
{
	int i;
	THEADER *temp;
	DATA ID;
	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Error in Operation");
		return(-1);
	}
	ID.ROWS=it->img.ROWS;
	ID.COLS=it->img.COLS;
	ID.ver=it->ver;


	temp=(THEADER *)malloc(sizeof(THEADER));
	if(temp==(THEADER *)0)
	{
		HIPL_ERRNO=-2;
		HIPL_Error("OUTPUT HEADER");
		return(-1);
	}

	i=HIPL_Initialise(temp,"",1,0,&ID);
	if(i<0)
	{
		HIPL_Error("Error in Initialisation");
		return(-1);
	}
	i=HIPL_AllocateMem(temp);
	if(i<0)
	{
		HIPL_Error("Error in Initialisation");
		return(-1);
	}
	temp->MROWS = it->MROWS;
	temp->MCOLS = it->MCOLS;
	temp->MORGX = it->MORGX;
	temp->MORGY = it->MORGY;

	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Morphology - Open operation");
		return(-1);
	}

	i=HIPL_BiErode(it,temp,Mask);
	if(i<0)
	{
		HIPL_Error("Error in Open - Erode");
		return(-1);
	}

	HIPL_Refresh(temp,0);

	i=HIPL_BiDilate(temp,ot,Mask);
	if(i<0)
	{
		HIPL_Error("Error in Open - Dilate");
		return(-1);
	}

	HIPL_Refresh(it,0);

	i=HIPL_Closeall(temp);
	if(i!=0)
	{
		HIPL_ERRNO=-21;
		HIPL_Error("Error in Closeall");
		return(-1);
	}
	
	i=HIPL_FreeMem(temp);
	if(i!=0)
	{
		HIPL_ERRNO=-21;
		HIPL_Error("Error in Freeimg Memory");
		return(-1);
	}
	return(0);
}

/****************************************************************
*	FUNCTION : HIPL_BiClose					*
*								*
*	INPUT	 : Input and  the Output headers		*
*		    the mask,number of rows to be processed	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Moves the mask on the array in the input     *
*		   header and places the result in the output	*
*		   array.This function is called repeatedly by	*
*		   HIPL_Medianmask function (for every Getpart)	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*			File : HIPL_Coreio.c			*
*								*	
****************************************************************/

int HIPL_BiClose(THEADER *it,THEADER *ot,int **Mask)
{
	int i;
	THEADER *temp;
	DATA ID;
	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Error in Operation");
		return(-1);
	}

	ID.ROWS=it->img.ROWS;
	ID.COLS=it->img.COLS;
	ID.ver=it->ver;


	temp=(THEADER *)malloc(sizeof(THEADER));
	if(temp==(THEADER *)0)
	{
		HIPL_ERRNO=-2;
		HIPL_Error("OUTPUT HEADER");
		return(-1);
	}

	i=HIPL_Initialise(temp,"",1,0,&ID);
	if(i<0)
	{
		HIPL_Error("Error in Initialisation");
		return(-1);
	}
	i=HIPL_AllocateMem(temp);
	if(i<0)
	{
		HIPL_Error("Error in Initialisation");
		return(-1);
	}

	temp->MROWS = it->MROWS;
	temp->MCOLS = it->MCOLS;
	temp->MORGX = it->MORGX;
	temp->MORGY = it->MORGY;
	if(it->P.psize<=it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}

	i=HIPL_BiDilate(it,temp,Mask);
	if(i<0)
	{
		HIPL_Error("Error in Open - Dilate");
		return(-1);
	}

	HIPL_Refresh(temp,0);

	i=HIPL_BiErode(temp,ot,Mask);
	if(i<0)
	{
		HIPL_Error("Error in Open - Erode");
		return(-1);
	}

	HIPL_Refresh(it,0);

	i=HIPL_Closeall(temp);
	if(i!=0)
	{
		HIPL_ERRNO=-21;
		HIPL_Error("Error in Closeall");
		return(-1);
	}
	i=HIPL_FreeMem(temp);
	if(i!=0)
	{
		HIPL_ERRNO=-21;
		HIPL_Error("Error in Freeimg Memory");
		return(-1);
	}
	return(0);
}

