#include "incl.h"
#define M_SCALE 20.0
#define M_SIZE 20
#define ORI_SCALE 40.0  
float dgauss(float x,float sigma);
float gauss(float x,float sigma);
float norm(float x,float y);
extern int HIPL_Initialise(THEADER *,char *,short,short,DATA *);
extern int HIPL_AllocateMem(THEADER *);
extern int HIPL_Getpart(THEADER *);
extern int HIPL_Putpart(THEADER *);
extern int HIPL_Closeall(THEADER *);
extern int HIPL_FreeMem(THEADER *);
extern int HIPL_Refresh(THEADER *,short);
extern int HIPL_ERRNO;
extern void HIPL_Error(char *);
extern int HIPL_Ffmaskimg(THEADER *, THEADER *,float **);
int hysteresis(THEADER *,int,int,DATA *);
int estimate_threshold(THEADER *mag,int *high,int *low,int wid);
int canny1(double s,THEADER *it,THEADER *mt,DATA *ID1,int *wid);
int nonmax_suppress(THEADER *dx,THEADER *dy,THEADER *mag);

float gauss(float x,float sigma)
{
return((float)exp((double)((-x*x)/(2*sigma*sigma))));
}
float meangauss(float x,float sigma)
{
float z;
z=(gauss(x,sigma)+gauss(x+0.5,sigma)+gauss(x-0.5,sigma))/3.0;
z=z/((float)M_PI*2.0*sigma*sigma);
return z;
}
float dgauss(float x,float sigma)
{
return((-x/(sigma*sigma))*gauss(x,sigma));
}

int HIPL_Canny(THEADER *it,THEADER *ot,double sigma,int lo,int hi)
{
THEADER *mt;
DATA *ID1;
int i,wid,max,j;
mt=(THEADER *)malloc(sizeof(THEADER));
ID1=(DATA *)malloc(sizeof(DATA));
ID1->ROWS=it->img.ROWS;
ID1->COLS=it->img.COLS;
ID1->ver=it->ver;
canny1(sigma,it,mt,ID1,&wid);
i=HIPL_Refresh(it,0);
i=HIPL_Refresh(ot,0);
mt->MROWS=0;
do {
	max=HIPL_Getpart(ot);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=0;i<max;i++)
	for(j=0;j<ot->img.COLS;j++)
	{
	ot->iarr[i][j]=0;
	}
	max=HIPL_Putpart(ot);
	if(max<0) {
	HIPL_Error("Error in Putpart");
	return(-1);
	}
  }while(ot->P.nleft);

i=HIPL_Refresh(mt,0);
i=HIPL_Refresh(ot,0);
if((hi<0)||(lo<0))
{
i=estimate_threshold(mt,&hi,&lo,wid);
}
i=HIPL_Refresh(mt,0);
i=hysteresis(mt,hi,lo,ID1);
i=HIPL_Refresh(mt,0);
mt->MROWS=0;
do {
	max=HIPL_Getpart(mt);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=wid;i<max-wid;i++)
	for(j=wid;j<mt->img.COLS-wid;j++)
	{
	ot->iarr[i][j]=mt->iarr[i][j];
	if(mt->iarr[i][j]!=255) ot->iarr[i][j]=0;
	}
	max=HIPL_Putpart(ot);
	if(max<0) {
	HIPL_Error("Error in Putpart");
	return(-1);
	}
  }while(ot->P.nleft);
i=HIPL_Closeall(mt);
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
i=HIPL_FreeMem(mt);
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}
free(mt);
free(ID1);
return 0;
}
float norm(float x,float y)
{
return((float)sqrt((double)(x*x+y*y)));
}
int canny1(double s,THEADER *it,THEADER *mt,DATA *ID1,int *wid)
{
float gau[M_SIZE],dgau[M_SIZE],z;
THEADER *dx,*dy,*gx,*gy;
float **maskg;
int i,j,n,width,WIDTH,max,val;
gx=(THEADER *)malloc(sizeof(THEADER));
gy=(THEADER *)malloc(sizeof(THEADER));
dx=(THEADER *)malloc(sizeof(THEADER));
dy=(THEADER *)malloc(sizeof(THEADER));
i=HIPL_Initialise(gx,NULL,1,1,ID1);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(gx);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_Initialise(dx,NULL,1,1,ID1);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(dx);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
for(i=0;i<M_SIZE;i++)
	{
	gau[i]=meangauss((float)i,s);
	if(gau[i]<0.005){
		width=i-1;
		break;
		}
	dgau[i]=dgauss((float)i,s);
	}
*wid=width;
n=2*width+1;
maskg=(float **)malloc(sizeof(float *));
maskg[0]=(float *)malloc(n*sizeof(float));
for(i=0;i<=width;i++)
{
maskg[0][i]=maskg[0][n-1-i]=gau[width-i];
}
WIDTH=width/2;
it->MROWS=gx->MROWS=dx->MROWS=1;
it->MCOLS=gx->MCOLS=dx->MCOLS=n;
it->MORGX=gx->MORGX=dx->MORGX=0;
it->MORGY=gx->MORGY=dx->MORGY=width;

do {
	max =HIPL_Getpart(it);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++)
			dx->farr[i][j]=(float)it->iarr[i][j];
	max=HIPL_Putpart(dx);
	if(max<0) {
	HIPL_Error("Error in Putpart");
	return(-1);
	}
	}while(dx->P.nleft);
HIPL_Refresh(dx,0);	
i=HIPL_Ffmaskimg(dx,gx,maskg);
HIPL_Refresh(dx,0);
HIPL_Refresh(gx,0);
free(maskg[0]);
free(maskg);

maskg=(float **)malloc(sizeof(float *));
maskg[0]=(float *)malloc(n*sizeof(float));
for(i=0;i<=width;i++)
{
maskg[0][i]=-dgau[width-i];
maskg[0][n-1-i]=dgau[width-i];
}
gx->MROWS=dx->MROWS=1;
gx->MCOLS=dx->MCOLS=n;
gx->MORGX=dx->MORGX=0;
gx->MORGY=dx->MORGY=width;
i=HIPL_Ffmaskimg(gx,dx,maskg);
free(maskg[0]);
free(maskg);
i=HIPL_Closeall(gx);
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
i=HIPL_FreeMem(gx);
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}

i=HIPL_Refresh(it,0);

i=HIPL_Initialise(gy,NULL,1,1,ID1);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(gy);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_Initialise(dy,NULL,1,1,ID1);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(dy);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
it->MROWS=gy->MROWS=dy->MROWS=n;
it->MCOLS=gy->MCOLS=dy->MCOLS=1;
it->MORGX=gy->MORGX=dy->MORGX=width;
it->MORGY=gy->MORGY=dy->MORGY=0;

maskg=(float **)malloc(n*sizeof(float *));
for(i=0;i<n;i++)
maskg[i]=(float *)malloc(sizeof(float));
for(i=0;i<=width;i++)
maskg[i][0]=maskg[n-1-i][0]=gau[width-i];

do {
	max =HIPL_Getpart(it);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++)
			dy->farr[i][j]=(float)it->iarr[i][j];
	max=HIPL_Putpart(dy);
	if(max<0) {
	HIPL_Error("Error in Putpart");
	return(-1);
	}
	}while(dy->P.nleft);
	
HIPL_Refresh(dy,0);
i=HIPL_Ffmaskimg(dy,gy,maskg);
HIPL_Refresh(dy,0);
HIPL_Refresh(gy,0);
for(i=0;i<=width;i++)
free(maskg[i]);
free(maskg);

maskg=(float **)malloc(n*sizeof(float *));
for(i=0;i<n;i++)
maskg[i]=(float *)malloc(sizeof(float));
for(i=0;i<=width;i++)
{
maskg[i][0]=-dgau[width-i];
maskg[n-1-i][0]=dgau[width-i];
}
gy->MROWS=dy->MROWS=n;
gy->MCOLS=dy->MCOLS=1;
gy->MORGX=dy->MORGX=width;
gy->MORGY=dy->MORGY=0;
i=HIPL_Ffmaskimg(gy,dy,maskg);
for(i=0;i<=width;i++)
free(maskg[i]);
free(maskg);
i=HIPL_Closeall(gy);
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
i=HIPL_FreeMem(gy);
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}

HIPL_Refresh(dy,0);
HIPL_Refresh(dx,0);

i=HIPL_Initialise(mt,NULL,1,0,ID1);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(mt);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}

do {
	max=HIPL_Getpart(dx);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	max=HIPL_Getpart(dy);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=0;i<max;i++)
	{
		for(j=0;j<dx->img.COLS;j++)
		{
		z = norm(dx->farr[i][j],dy->farr[i][j]);
		val=mt->iarr[i][j]=(int)z*M_SCALE;
		}
	}
	max=HIPL_Putpart(mt);
	if(max<0) {
	HIPL_Error("Error in Putpart");
	return(-1);
	}
}while(mt->P.nleft);
HIPL_Refresh(dy,0);
HIPL_Refresh(dx,0);
HIPL_Refresh(mt,0);
i=nonmax_suppress(dx,dy,mt);
i=HIPL_Closeall(dx);
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
i=HIPL_FreeMem(dx);
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}
i=HIPL_Closeall(dy);
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
i=HIPL_FreeMem(dy);
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}
return 0;
}

int nonmax_suppress(THEADER *dx,THEADER *dy,THEADER *mag)
{
int i,j,max;
float xx,yy,g1,g2,g3,g4,g,xc,yc,temp1,temp2;
do{
	max = HIPL_Getpart(dx);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	max = HIPL_Getpart(dy);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=1;i<max-1;i++)
		{
		for(j=1;j<dx->img.COLS-1;j++)
			{
			mag->iarr[i][j] = 0;
			xc = dx->farr[i][j];	
			yc = dy->farr[i][j];
			if((fabs(xc)<0.01)&&(fabs(yc)<0.01)) continue;
			g = norm(xc,yc);
			
			if(fabs(yc)>fabs(xc))
				{
				xx=fabs(xc)/fabs(yc);
				yy= 1.0;
				g2 = norm(dx->farr[i-1][j],dy->farr[i-1][j]);
				g4 = norm(dx->farr[i+1][j],dy->farr[i+1][j]);
				if(xc*yc>0.0)
					{
					g3 = norm(dx->farr[i+1][j+1],dy->farr[i+1][j+1]);
					g1 = norm(dx->farr[i-1][j-1],dy->farr[i-1][j-1]);
					}
				else
					{
					g3 = norm(dx->farr[i+1][j-1],dy->farr[i+1][j-1]);
					g1 = norm(dx->farr[i-1][j+1],dy->farr[i-1][j+1]);
					}
				}
			else
				{
				xx=fabs(yc)/fabs(xc);
				yy= 1.0;
				g2 = norm(dx->farr[i][j+1],dy->farr[i][j+1]);
				g4 = norm(dx->farr[i][j-1],dy->farr[i][j-1]);
				if(xc*yc>0.0)
					{
					g3 = norm(dx->farr[i-1][j-1],dy->farr[i-1][j-1]);
					g1 = norm(dx->farr[i+1][j+1],dy->farr[i+1][j+1]);
					}
				else
					{
					g3 = norm(dx->farr[i-1][j+1],dy->farr[i-1][j+1]);
					g1 = norm(dx->farr[i+1][j-1],dy->farr[i+1][j-1]);
					}
				}
			temp1=xx*g1+(yy - xx)*g2;
			temp2=xx*g3+(yy - xx)*g4;
			if(((g+0.05-temp1)>0) &&((g+0.05-temp2 )>0))
				{
					if((g*M_SCALE) <= 255.0)
					mag->iarr[i][j] =(int)(g*M_SCALE);
					else
					mag->iarr[i][j] = 255;
				}
			else
				{
					mag->iarr[i][j] = 0;
				}
			
			}
		}
		max = HIPL_Putpart(mag);
		if(max<0) {
		HIPL_Error("Error in Putpart");
		return(-1);
		}
	}while(mag->P.nleft);

return 0;
}

int estimate_threshold(THEADER *mag,int *high,int *low,int wid)
{
	int i,j,k,hist[256],count,max;
	for(i=0;i<256;i++) hist[i]=0;
	mag->MROWS=wid+1;
	mag->MCOLS=wid+1;
	mag->MORGX=wid;
	mag->MORGY=wid;

	do{
		max = HIPL_Getpart(mag);
		if(max<0) {
		HIPL_Error("Error in Getpart");
		return(-1);
		}
		for(i=wid;i<max-wid;i++)
			for(j=wid;j<mag->img.COLS-wid;j++)
				hist[mag->iarr[i][j]]++;
	}while(mag->P.nleft);
	j=mag->img.ROWS;
	if(j<mag->img.COLS) j=mag->img.COLS;
	j*=0.9;
	k=255;
	count=hist[255];
	while(count<j)
	{
	k--;
	if(k<0) break;
	count+=hist[k];
	}
	*high=k;
	i=0;
	while(hist[i]==0) i++;
	*low=(*high+1)/2.0;
	HIPL_Refresh(mag,0);
	return 0;
}


int hysteresis(THEADER *mag,int high,int low,DATA *ID)
{
int i,j,flag,l,k,max;
THEADER *temp;
temp=(THEADER *)malloc(sizeof(THEADER));
i=HIPL_Initialise(temp,NULL,1,0,ID);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
i=HIPL_AllocateMem(temp);
if(i<0) {
HIPL_Error("Error in Initialisation");
return -1;
}
temp->MROWS=0;
mag->MROWS=0;
HIPL_Refresh(temp,0);
HIPL_Refresh(mag,0);
do{
	max=HIPL_Getpart(mag);
	if(max<0) {
	HIPL_Error("Error in Getpart");
	return(-1);
	}
	for(i=0;i<max;i++)
		for(j=0;j<mag->img.COLS;j++)
		{
		if(mag->iarr[i][j]>=high)
 		temp->iarr[i][j]=255;
		else
		if(mag->iarr[i][j]<=low) 
		temp->iarr[i][j]=0;
		else
		{
		temp->iarr[i][j]=mag->iarr[i][j];
		}
		}
		max=HIPL_Putpart(temp);
		if(max<0) {
		HIPL_Error("Error in Putpart");
		return(-1);
		}
}while(temp->P.nleft);
temp->MROWS=mag->MROWS=3;
temp->MCOLS=mag->MCOLS=3;
temp->MORGX=mag->MORGX=1;
temp->MORGY=mag->MORGY=1;
HIPL_Refresh(temp,0);
HIPL_Refresh(mag,0);
do{
	flag=0;
	do{
		max=HIPL_Getpart(temp);
		if(max<0) {
		HIPL_Error("Error in Getpart");
		return(-1);
		}
		for(i=1;i<max-1;i++)
			for(j=1;j<temp->img.COLS-1;j++)
			{
			if(temp->iarr[i][j]==255)
				for(l=-1;l<=1;l++)
				for(k=-1;k<=1;k++)
				if((temp->iarr[i+l][j+k]>low)&&(temp->iarr[i+l][j+k]<high))
					{
					mag->iarr[i+l][j+k]=255;
					flag=1;
					}
			}
		max=HIPL_Putpart(mag);
		if(max<0) {
		HIPL_Error("Error in Putpart");
		return(-1);
		}
	}while(mag->P.nleft);
	if(flag){
		HIPL_Refresh(temp,0);
		HIPL_Refresh(mag,0);
		do {
			max=HIPL_Getpart(mag);
			if(max<0) {
			HIPL_Error("Error in Getpart");
			return(-1);
			}
			for(i=0;i<max;i++)
				for(j=0;j<mag->img.COLS;j++)
					temp->iarr[i][j]=mag->iarr[i][j];
			max=HIPL_Putpart(temp);
			if(max<0) {
			HIPL_Error("Error in Putpart");
			return(-1);
			}
	           }while(temp->P.nleft);
		}
HIPL_Refresh(temp,0);
HIPL_Refresh(mag,0);
}while(flag);
i=HIPL_Closeall(temp);	
if(i<0) {
HIPL_Error("Error in Closeall");
return(-1);
}
HIPL_FreeMem(temp);	
if(i<0) {
HIPL_Error("Error in Freeing Memory");
return(-1);
}
free(temp);
return 0;
}

