#include "incl.h"

int HIPL_Gaussavg(THEADER *it,THEADER *ot,int sigma);
int Gauss_Avg(THEADER *T1,THEADER *T2,float *G,int sigma);
int Comp_GMask(float *GMask,int sigma);

extern int HIPL_Initialise(THEADER *t,char *name,short i,short j,DATA *ID);
extern int HIPL_AllocateMem(THEADER *);
extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_ERRNO;
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_FreeMem(THEADER *T);
extern int HIPL_Refresh(THEADER *T,int i);
extern void HIPL_Error(char *a);

/****************************************************************
*	FUNCTION : HIPL_Gaussavg				*
*								*
*	INPUT	 : source and target file names			*
*								*
*	OUTPUT	 : 0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Applies Gaussian mask on the input image     *
*		   and stores the result in the result file	*	
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*	
*		     5)Readint					*
*			File : usrio.c				*
*								*
****************************************************************/

int HIPL_Gaussavg(THEADER *it,THEADER *ot,int sigma)
{
	THEADER *gt;
	float *GMask;
	int i,j,k,msk;
	DATA GID;
	gt=(THEADER *)malloc(sizeof(THEADER));
	if(gt==(THEADER *)0)
	{
		HIPL_ERRNO=-2;
		HIPL_Error("INPUT HEADER");
		return(-1);
	}

	GID.ROWS=it->img.ROWS;
	GID.COLS=it->img.COLS;
	GID.ver=it->ver;


	i=HIPL_Initialise(gt,"",1,1,&GID);
	if(i<0)
	{
		HIPL_Error("Initialising output");
		return(-1);
	}
	i=HIPL_AllocateMem(gt);
	if(i<0)
	{
		HIPL_Error("Initialising output");
		return(-1);
	}
	msk=(6*sigma)+1;
	if(it->P.psize<=msk)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Copy operation");
		return(-1);
	}
	

	GMask=(float *)malloc(msk*sizeof(float));
	if(GMask==(float *)0)
	{
		HIPL_ERRNO=-9;
		HIPL_Error("Gaussian Average");
		return(-1);
	}

	i=Comp_GMask(GMask,sigma);
	i=Gauss_Avg(it,gt,GMask,sigma);
	gt->MROWS=0;
	gt->MCOLS=0;
	i=HIPL_Refresh(gt,0);

	do
	{
		i=HIPL_Getpart(gt);
		if(i<0)
		{
			HIPL_Error("Getpart in copy");
			return(i);
		}


		for(j=0;j<i;j++)
			for(k=0;k<gt->img.COLS;k++)
				ot->iarr[j][k]=(int)gt->farr[j][k];

		i=HIPL_Putpart(ot);

		if(i<0)
		{
			HIPL_Error("Putpart in copy");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);


	i=HIPL_Closeall(gt);
	if(i!=0)
	{
		HIPL_Error("Gaussian average - output header1");
		return(-1);
	}
	i=HIPL_FreeMem(gt);
	if(i!=0)
	{
		HIPL_Error("Gaussian average - output header1");
		return(-1);
	}
	free(GMask);
	return(0);
}

/****************************************************************
*	FUNCTION : Comp_GMask					*
*								*	
*	INPUT	 : Float array (Guassian mask)			*
*								*
*	OUTPUT	 : 0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Computes Gaussian mask    			*
*								*
*   EXTERNAL CALLS : NONE					*
*								*
****************************************************************/

int Comp_GMask(float *GMask,int sigma)
{
	float gmax,a,b,c,d,e,pi;
	int i,j;
	j=(int)sigma;
	pi=(float)M_PI;
	a=(float)sigma;
	b=(float)sqrt(2*pi);
	c=b*a;
	d=2*a*a;
	gmax=(float)1.0/c;

	for(i=1;i<=(3*j);i++)
	{
		e=(float)((-1)*i*i);
		GMask[(3*j)-i]=(float)gmax*(float)exp(e/d);
		GMask[(3*j)+i]=GMask[(3*j)-i];
	}

	GMask[3*j]=gmax;
	return(0);

}

/****************************************************************
*	FUNCTION : Gauss_Avg					*
*								*
*	INPUT	 : Input and the  result headers,name of the	*
*		   input image file Gaussian mask		*
*			and the value of sigma			*
*								*
*	OUTPUT	 : 0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Applies Gaussian mask on the temporary file  *
*		   in the input header and stores the result in	*
*		   the temporary file in the result header	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_Error				*
*			File :HIPL_Coreio.c			*
*		     						*
*								*
****************************************************************/

int Gauss_Avg(THEADER *it,THEADER *gt,float *GMask,int sigma)

{

	int i,j,fus,hs,max,k;
	float **temp,t1,t2;

	fus=(6*sigma)+1;
	hs=fus/2;
	it->MROWS=gt->MROWS=fus;
	it->MORGX=gt->MORGX=hs;
	temp=(float **)malloc(it->P.psize*sizeof(float *));
	if(temp==(float **)0)
	{
		HIPL_ERRNO=-5;
		HIPL_Error("Temporary array in Gaussian Average");
		return(-1);
	}

	for(i=0;i<it->P.psize;i++)
	{
		temp[i]=(float *)malloc(it->img.COLS * sizeof(float));
		if(temp[i]==(float *)0)
		{
			HIPL_ERRNO=-5;
			HIPL_Error("Temporary array in Gaussian Average");
			return(-1);
		}
	}	
	

	do
	{
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Getpart in Gaussian Average");
			return(i);
		}

		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				if((j<hs)||(j>=(it->img.COLS-hs)))
					temp[i][j]=(float) it->iarr[i][j];
				else
				{
					t1=0;
					for(k=-hs;k<=hs;k++)
					{
						t2=GMask[k+hs]*((float)it->iarr[i][j+k]); 
						t1=t1+t2;
					}
					temp[i][j]=t1;
				}
			}
		}


		for(i=0;i<max;i++)
		{
			for(j=0;j<it->img.COLS;j++)
			{
				if((i<hs)||(i>=(max-hs)))
					gt->farr[i][j]=temp[i][j];
				else
				{
					t1=0;
					for(k=-hs;k<=hs;k++)
					{
						t2=GMask[k+hs]*(temp[i+k][j]);
						t1=t1+t2;
					}
					gt->farr[i][j]=t1;
				}
			}
		}


		i=HIPL_Putpart(gt);
		if(i<0)
		{
			HIPL_Error("Putpart in Gaussian Average");
			return(i);
		}


		if(gt->P.nleft==0)
			i=-1;
	}while(i>=0);

	for(i=0;i<it->P.psize;i++)
		free(temp[i]);

	free(temp);

	return(0);

}

