#include "incl.h"

extern int HIPL_Initialise(THEADER *t,char *name,short i,short j,DATA *ID);
extern int HIPL_AllocateMem(THEADER *t);
extern int HIPL_Getpart(THEADER *t);
extern int HIPL_Putpart(THEADER *t);
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_FreeMem(THEADER *t);
extern int HIPL_Refresh(THEADER *T,int flag);
extern void HIPL_Error(char *msg);
extern int HIPL_ERRNO;

int HIPL_Str(THEADER *it,THEADER *ot,long int hist[257],int gmax,int gmin);
int HIPL_Lstretch(THEADER *it,THEADER *ot);
int HIPL_Equalise(THEADER *it,THEADER *ot,long int hist[257]);
int HIPL_ImgData(THEADER *it,long int hist[257],DATA *ID);
int HIPL_FindHistogram(THEADER *it,long int hist[257]);

/****************************************************************
*	FUNCTION : Hist_str					*
*								*
*	INPUT	 : Input header,Output header  Maximum and	*
*		    Minimum gray scale values in the image	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Stretches the histogram to the user specifed	*
*			gray levels				*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
*								*
****************************************************************/

int HIPL_Str(THEADER *it,THEADER *ot,long int hist[257],int gmax,int gmin)
{
	int i,j,k,tmax,tmin;
	float val,val1,val2,value;
	for(i=0;i<256;i++)
	{
		if(hist[i]!=0)
		{
			tmin=i;
			break;
		}
	}
	for(i=255;i>=0;i--)
	{
		if(hist[i]!=0)
		{
			tmax=i;
			break;
		}
	}
	if(tmax<=tmin)
		return(0);
	if(gmax>255)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	if(gmin<0)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	if(gmax<=gmin)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	val1=(float)(gmax-gmin);
	val2=(float)(tmax-tmin);
	value=val1/val2;
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Histogram Stretch");
			return(i);
		}
		for(j=0;j<i;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				val1=(float)(it->iarr[j][k]-tmin);
				val2=value*val1;
				val=val2+((float)gmin);
				ot->iarr[j][k]=(int)val;
				if(ot->iarr[j][k]>255)
					ot->iarr[j][k]=255;
				if(ot->iarr[j][k]<0)
					ot->iarr[j][k]=0;
			}
		}
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Histogram Stretch");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}

/****************************************************************
*	FUNCTION : Hist_equ					*
*								*
*	INPUT	 : Input header,Output header  Maximum and	*
*		    Minimum gray scale values in the image	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Stretches the histogram to the user specifed	*
*			gray levels				*
*								*
*   EXTERNAL CALLS : 1)HIPL_Getpart				*
*		     2)HIPL_Putpart				*
*		     3)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
****************************************************************/

int HIPL_Equalise( THEADER *it,THEADER *ot,long int hist[257])
{
	int i,j,k,level=256,m,max;
	float cont,t1,t2;
	float area=0,sum_hist[257];
	for(i=0;i<256;i++)
	sum_hist[i]=0;
	for(i=0;i<256;i++)
	{
		area=area+(float)hist[i];
		sum_hist[i]=area;
	}
	area=((float)it->img.ROWS*(float)it->img.COLS);
	t2=(float)level;
	t1=(float)area;
	if(t1==0)
	{
		HIPL_ERRNO=-20;
		return(-1);
	}
	cont=t2/t1;
	do
	{
		max=0;
		max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Histogram Equalise");
			return(i);
		}
		for(j=0;j<max;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				m=it->iarr[j][k];
				t1=(float)sum_hist[m];
				t2=t1*cont;
				ot->iarr[j][k]=(int)t2;
				if(ot->iarr[j][k]>255)
					ot->iarr[j][k]=255;
				if(ot->iarr[j][k]<0)
					ot->iarr[j][k]=0;
			}
		}
		max=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Histogram Equalise");
			return(i);
		}
		if(ot->P.nleft==0)
			max=-1;
	}while(max>=0);
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	return(0);
}

/****************************************************************
*	FUNCTION : Image_data					*
*								*
*	INPUT	 :  The Input Header				*
*								*
*	OUTPUT	 : 0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Calculates the average gray level and the 	*
*		            standard deviation of gray levels	*
*								*
*  EXTERNAL CALLS :  1)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
*		     2)Readint					*
*		        File :userio.c				*
*								*
****************************************************************/

int HIPL_ImgData(THEADER *it,long int hist[257],DATA *ID)
{
	int i,j,k,mean,dev,tmax,tmin;
	double sum=0,sum_squares=0,avg,x,y,num_pixels,avg_squares,deviation;
	it->MROWS=0;
	for(i=0;i<256;i++)
	{
		if(hist[i]!=0)
		{
			tmin=i;
			break;
		}
	}
	for(i=255;i>=0;i--)
	{
		if(hist[i]!=0)
		{
			tmax=i;
			break;
		}
	}
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Finding Image Data");
			return(i);
		}
		for(j=0;j<i;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				x=(double)it->iarr[j][k];
				sum=sum+x;
				y=x*x;
				sum_squares=sum_squares+y;
			}
		}
	}while(i>0);
	num_pixels=(double)(it->img.ROWS)*(it->img.COLS);
	avg= sum/num_pixels;
	mean=(int)avg;
	avg_squares=sum_squares/num_pixels;
	deviation=avg_squares-(avg*avg);
	dev=(int)sqrt(deviation);
	ID->maxg=tmax;
	ID->ming=tmin;
	ID->mean=mean;
	ID->dev=dev;
	HIPL_Refresh(it,0);
	return(0);
}

/****************************************************************
*	FUNCTION : Image_data					*
*								*
*	INPUT	 :  The Input Header				*
*								*
*	OUTPUT	 : 0  on success				*
*		   -1 on failure				*
*								*
*    DESCRIPTION : Calculates the average gray level and the 	*
*		            standard deviation of gray levels	*
*								*
*  EXTERNAL CALLS :  1)HIPL_Error				*
*		  File :HIPL_Coreio.c				*
*								*
*		     2)Readint					*
*		        File :userio.c				*
*								*
****************************************************************/

int HIPL_FindHistogram(THEADER *it,long int hist[257])
{
	long int x;
	int i,j,k;
	do
	{
		x=0;
		i=HIPL_Getpart(it);
		if(i<0)
			return(i);
			
		for(j=0;j<i;j++)
		{
			for(k=0;k<it->img.COLS;k++)
			{
				x=it->iarr[j][k];
				if(x>255)
					x=255;
				if(x<0)
					x=0;
				hist[x]++;
			}
		}
	}while(i>0);
	HIPL_Refresh(it,0);
	return(0);
}

/****************************************************************/

int HIPL_Lstretch(THEADER *it,THEADER *ot)
{
	int i,j,k,x,L[256],temp;
	double base;
	DATA *ID;
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=it->img.ROWS;
	ID->COLS=it->img.COLS;
	ID->ver=it->ver;
	
	i=CIPL_FindHistogram(it,ID,256,1);
		if(i<0) return -1;
	//HIPL_FindHistogram(it,ID->hist);
	HIPL_Refresh(it,0);
	HIPL_ImgData(it,ID->hist,ID);
	HIPL_Refresh(it,0);
	if( ID->maxg > 1)
		base=pow(10,log10((double)(ID->maxg))/255.0);
	
	switch(ID->maxg) {
		case 0 : 
			L[0] = 255;
			break;
		case 1 :
			L[1] = 255;
			L[0] = 0;
			break;
		default :
			for(i=0;i<256;i++) {
			if(i==0) {
			temp = 0;
			} else {
			temp=(int)(log10((double)(i))/log10(base));
			}
			if(temp > 255) temp = 255;
			L[i]=temp;
			}
			break;
		}
	HIPL_Refresh(it,0);	
	HIPL_Refresh(ot,0);
	
	do
	{
		x=0;
		i=HIPL_Getpart(it);
		if(i<0)
			return(i);
		for(j=0;j<i;j++)
			for(k=0;k<it->img.COLS;k++)
				ot->iarr[j][k]=L[it->iarr[j][k]];
		i=HIPL_Putpart(ot);
		if(i<0)
			return(i);
	}while(ot->P.nleft);
	
	free(ID);
	return(0);
}
