#include "incl.h"

extern int  HIPL_Getpart(THEADER *T);
extern int HIPL_Putpart(THEADER *T);
extern void HIPL_Error(char *msg);
extern int HIPL_Refresh(THEADER *T,int flag);
extern int HIPL_Initialise(THEADER *,char *,short,short,DATA *);
extern int HIPL_AllocateMem(THEADER *);
extern int HIPL_Closeall(THEADER *);
extern int HIPL_FreeMem(THEADER *);

int HIPL_Thinning(THEADER *it,THEADER *ot);
int HIPL_Prewitts(THEADER *it,THEADER *ot);
int HIPL_Roberts(THEADER *it,THEADER *ot);
int HIPL_Kirsch(THEADER *it,THEADER *ot);
int HIPL_IIRMaskimg(THEADER *it,THEADER *ot,float **a,float **b);
int HIPL_Sobel(THEADER *it,THEADER *ot);
int Move_IIRMask(THEADER *it,THEADER *ot,float **a,float **b,int max);
int FMove_fMask(THEADER *it,THEADER *ot,float **Mask,int max);
int Roberts_Move(THEADER *it,THEADER *ot,int max);
int Prewitts_Move(THEADER *it,THEADER *ot,int max);
int Sobel_Move(THEADER *it,THEADER *ot,int max);
int Kirsch_Move(THEADER *it,THEADER *ot,int max);
int Move_Mask(THEADER *it,THEADER *ot,int **Mask,int max,float reci);
int HIPL_Maskimg(THEADER *it,THEADER *ot,int **Mask,double factor);
int Move_fMask(THEADER *it,THEADER *ot,double **Mask,int max,float reci);
int HIPL_Fmaskimg(THEADER *it,THEADER *ot,double **Mask,double factor);
int Move_Median(THEADER *it,THEADER *ot,int max);
int HIPL_Medianmask(THEADER *it,THEADER *ot);
int Sort(int *Temp,int size);
extern int HIPL_ERRNO;

/****************************************************************
*	FUNCTION : HIPL_Maskimg					*
*								*
*	INPUT	 : source and target file names			*
*		    The mask,mask size and the dividing factor	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Applies the mask on the source file and	*
*			places the result in the target file	*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*			File : HIPL_Coreio.c			*
*								*
****************************************************************/

int HIPL_Maskimg(THEADER *it,THEADER *ot,int **Mask,double factor)
{

	float reci;
	int i;
	if(factor==0.0)
	{
		HIPL_ERRNO=-19;
		HIPL_Error("Mask operation");
		return(-1);
	}

	reci=(float)(1/(float)factor);
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}

	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Move_Mask(it,ot,Mask,i,reci);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);

}
int HIPL_Fmaskimg(THEADER *it,THEADER *ot,double **Mask,double factor)
{

	float reci;
	int i;
	if(factor==0)
	{
		HIPL_ERRNO=-19;
		HIPL_Error("Mask operation");
		return(-1);
	}

	reci=(float)(1/(float)factor);
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}

	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Move_fMask(it,ot,Mask,i,reci);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);

}

int HIPL_Ffmaskimg(THEADER *it,THEADER *ot,float **Mask)
{

	int i;
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}

	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		FMove_fMask(it,ot,Mask,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);

}
int HIPL_IIRMaskimg(THEADER *it,THEADER *ot,float **a,float **b)
{
	int i;
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Move_IIRMask(it,ot,a,b,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);
}

int HIPL_Sobel(THEADER *it,THEADER *ot)
{
	int i;
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Sobel_Move(it,ot,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);
}

int HIPL_Kirsch(THEADER *it,THEADER *ot)
{

	int i;

	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}

	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Kirsch_Move(it,ot,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);
}

int HIPL_Roberts(THEADER *it,THEADER *ot)
{
	int i;
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Roberts_Move(it,ot,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);

}
int HIPL_Prewitts(THEADER *it,THEADER *ot)
{
	int i;
	if(it->P.psize<it->MROWS)
	{
		HIPL_ERRNO=-17;
		HIPL_Error("Mask operation");
		return(-1);
	}
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}

		Prewitts_Move(it,ot,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	HIPL_Refresh(ot,0);
	return(0);
}

/****************************************************************
*	FUNCTION : Move_Mask					*
*								*
*	INPUT	 : Input and  the Output headers		*
*		    the mask,number of rows to be processed	*
*		     and the reciprocal of the factor		*
*								*	
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Moves the mask on the array in the input     *
*		   header and places the result in the output	*
*		   array.This function is called repeatedly by	*
*		    HIPL_Maskimg function ie for every Getpart	*
*								*
*   EXTERNAL CALLS : None					*
*								*
****************************************************************/


int Move_Mask(THEADER *it,THEADER *ot,int **Mask,int max,float reci)
{
	int i,j,k,l,m,n;
	long int sum;
	float tem,t;
	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum=0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				sum=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							sum=sum+(Mask[k][l]*it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				tem=(float)(sum);
				t=tem*reci;
				ot->iarr[i][j]=abs(t);
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}

int Move_fMask(THEADER *it,THEADER *ot,double **Mask,int max,float reci)
{
	int i,j,k,l,m,n;
	double sum;
	double t;
	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum=0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				sum=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							sum=sum+(Mask[k][l]*(double)it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				t=sum*reci;
				ot->iarr[i][j]=(int)fabs(t);
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}

int Move_IIRMask(THEADER *it,THEADER *ot,float **a,float **b,int max)
{
	int i,j,k,l,m,n;
	float  sum;
	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
			ot->farr[i][j]=0.0;
		for(j=0;j<it->img.COLS;j++)
		{
			sum=0.0;
			m=-it->MORGY;
			n=-it->MORGX;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)	
				ot->farr[i][j]=0;//it->iarr[i][j];
				else
				ot->farr[i][j]=0;//it->farr[i][j];
			}
			else
			{
				sum=0.0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
					
					if(it->arrtype==0)
					{
					ot->farr[i][j]+=a[it->MROWS-k-1][it->MCOLS-l-1]*(float)it->iarr[i+n][j+m];
					if((it->MROWS+it->MCOLS-k-l-2)==0) continue;
					ot->farr[i][j]-=b[it->MROWS-k-1][it->MCOLS-l-1]*(float)ot->farr[i+n][j+m];
					}
				else
					{
					ot->farr[i][j]+=a[it->MROWS-k-1][it->MCOLS-l-1]*(float)it->farr[i+n][j+m];
					if((it->MROWS+it->MCOLS-k-l-2)==0) continue;
					ot->farr[i][j]-=b[it->MROWS-k-1][it->MCOLS-l-1]*(float)ot->farr[i+n][j+m];
					}
						m++;
					}
					m=-it->MORGY;
					n++;
				}
			}
		}
	}
	return(0);
}

int FMove_fMask(THEADER *it,THEADER *ot,float **Mask,int max)
{
	int i,j,k,l,m,n;
	float  sum;
	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum=0.0;
			m=-it->MORGY;
			n=-it->MORGX;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)	
				ot->farr[i][j]=it->iarr[i][j];
				else
				ot->farr[i][j]=it->farr[i][j];
			}
			else
			{
				sum=0.0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
					
				if(it->arrtype==0)	
				sum=sum+(Mask[k][l]*(float)it->iarr[i+n][j+m]);
				else
				sum=sum+(Mask[k][l]*(float)it->farr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				ot->farr[i][j]=(float)sum;
			}
		}
	}
	return(0);
}

int Sobel_Move(THEADER *it,THEADER *ot,int max)
{
	int i,j,k,l,m,n;
	long int sum1,sum2;
	double tem,t;
	int Mask1[3][3] = { {-1,-2,-1},
			    { 0, 0, 0},
			    { 1, 2 ,1}};

	int Mask2[3][3] = { {-1, 0, 1},
			    {-2, 0, 2},
			    {-1, 0 ,1}};

	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum1=0;
			sum2=0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				sum1=0;
				sum2=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							sum1=sum1+(Mask1[k][l]*it->iarr[i+n][j+m]);
							sum2=sum2+(Mask2[k][l]*it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				tem = sqrt((double)(sum1*sum1)+(double)(sum2*sum2));
				t=abs(tem);
				ot->iarr[i][j]=(int)t;
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}
int Kirsch_Move(THEADER *it,THEADER *ot,int max)
{
	int i,j,k,l,m,n,x;
	long int sum[8];
	double t;
	int Mask[8][3][3] = { { { 5, 5, 5},{-3,0,-3},{-3,-3,-3}},
			      { {5, 5,-3},{5,0,-3},{-3,-3,-3}},
			      { {5,-3,-3},{5,0,-3},{5,-3,-3}},
			      { {-3,-3,-3},{5, 0,-3},{5,5,-3}},
			      { {-3,-3,-3},{-3,0,-3},{5,5,5}},
			      { {-3,-3,-3},{-3,0,5},{-3,5,5}},
			      { {-3,-3,5},{-3,0,5},{-3,-3,5}},
			      { {-3,5,5},{-3,0,5},{-3,-3,-3}}};
					

	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			for(x=0;x<8;x++) sum[x] = 0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				for(x=0;x<8;x++) sum[x]=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							for(x=0;x<8;x++)
							sum[x]=sum[x]+(Mask[x][k][l]*it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				for(x=0;x<8;x++)
				sum[x]=abs(sum[x]);
				t=sum[0];
				for(x=1;x<8;x++)
				if(sum[x]>t) t= sum[x];
				ot->iarr[i][j]=(int)t;
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}

int Roberts_Move(THEADER *it,THEADER *ot,int max)
{
	int i,j,k,l,m,n;
	long int sum1,sum2;
	double tem,t;
	int Mask1[2][2] = {{1,0},{0,-1}};

	int Mask2[2][2] = {{0,1},{-1,0}};

	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum1=0;
			sum2=0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				sum1=0;
				sum2=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							sum1=sum1+(Mask1[k][l]*it->iarr[i+n][j+m]);
							sum2=sum2+(Mask2[k][l]*it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				tem = sqrt((double)(sum1*sum1)+(double)(sum2*sum2));
				t=abs(tem);
				ot->iarr[i][j]=(int)t;
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}

int Prewitts_Move(THEADER *it,THEADER *ot,int max)
{
	int i,j,k,l,m,n;
	long int sum1,sum2;
	double tem,t;
	int Mask1[3][3] = { {-1,-1,-1},
			    { 0, 0, 0},
			    { 1, 1 ,1}};

	int Mask2[3][3] = { {-1, 0, 1},
			    {-1, 0, 1},
			    {-1, 0 ,1}};

	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			sum1=0;
			sum2=0;
			m=-it->MORGX;
			n=-it->MORGY;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				sum1=0;
				sum2=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
							sum1=sum1+(Mask1[k][l]*it->iarr[i+n][j+m]);
							sum2=sum2+(Mask2[k][l]*it->iarr[i+n][j+m]);
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				tem = sqrt((double)(sum1*sum1)+(double)(sum2*sum2));
				t=abs(tem);
				ot->iarr[i][j]=(int)t;
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	return(0);
}

/****************************************************************
*	FUNCTION : HIPL_Medianmask				*
*								*
*	INPUT	 : source and target file names and mask size	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Applies the median filter on the source file * 
*			and places the result in the target file*
*								*
*   EXTERNAL CALLS : 1)HIPL_Initialise				*
*		     2)HIPL_Getpart				*
*		     3)HIPL_Putpart				*
*		     4)HIPL_StoreResult				*
*		     5)HIPL_Error				*
*		     6)HIPL_Closeall				*
*			File : HIPL_Coreio.c			*
*								*
****************************************************************/

int HIPL_Medianmask(THEADER *it,THEADER *ot)
{
	int i;
	do
	{
		i=HIPL_Getpart(it);
		if(i<0)
		{
			HIPL_Error("Error in Getpart");
			return(i);
		}
		Move_Median(it,ot,i);
		i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
		if(ot->P.nleft==0)
			i=-1;
	}while(i>=0);
	return(0);
}

/****************************************************************
*	FUNCTION : Move_Median					*
*								*
*	INPUT	 : Input and  the Output headers		*
*		    the mask,number of rows to be processed	*
*								*
*	OUTPUT	 : 0 on success					*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Moves the mask on the array in the input     *
*		   header and places the result in the output	*
*		   array.This function is called repeatedly by	*
*		   HIPL_Medianmask function (for every Getpart)	*
*								*
*   EXTERNAL CALLS : None					*
*								*	
****************************************************************/

int Move_Median(THEADER *it,THEADER *ot,int max)
{
	int i,j,k,l,m,n,*temp,t,sq,c;
	sq=it->MROWS*it->MCOLS;
	temp=(int *)malloc(sq*sizeof(int));
	if(temp==(int *)0)
	{
		HIPL_ERRNO=-5;
		HIPL_Error("Median Operation");
		return(-1);
	}
	for(i=0;i<max;i++)
	{
		for(j=0;j<it->img.COLS;j++)
		{
			m=-it->MORGY;
			n=-it->MORGX;
			if((i<it->MORGX)||((max-i)<(it->MROWS-it->MORGX))||(j<it->MORGY)||((it->img.COLS-j)<(it->MCOLS-it->MORGY)))
			{
				if(it->arrtype==0)
					ot->iarr[i][j]=it->iarr[i][j];
			}
			else
			{
				c=0;
				for(k=0;k<it->MROWS;k++)
				{
					for(l=0;l<it->MCOLS;l++)
					{
						if(it->arrtype==0)
						temp[c]=it->iarr[i+n][j+m];
						c++;
						m++;
					}
					m=-it->MORGY;
					n++;
				}
				t=Sort(temp,sq);
				ot->iarr[i][j]=(int)t;
				if(ot->iarr[i][j]>255)
					ot->iarr[i][j]=255;
				if(ot->iarr[i][j]<0)
					ot->iarr[i][j]=0;
			}
		}
	}
	free((char *)temp);
	return(0);
}

/****************************************************************
*	FUNCTION : Sort						*
*								*
*	INPUT	 :Array for which the median is to be found	*
*		    and the array size				*
*								*
*	OUTPUT	 : Median on success				*
*		   negative value on failure			*
*								*
*    DESCRIPTION : Finds the median of the neighbourhood which  *
*		     is given as a linear array			*
*								*
*   EXTERNAL CALLS : None					*
*								*
****************************************************************/

int Sort(int *Temp,int size)
{
	int i,j,m,x,flag=0;
	for(i=0;i<size;i++)
	{
		for(j=0;j<size-1;j++)
		{
			if(Temp[j]>Temp[j+1])
			{
				flag=1;
				x=Temp[j];
				Temp[j]=Temp[j+1];
				Temp[j+1]=x;
			}
		}
		if(flag==0)
				break;
		else flag=0;
	}
	m=(Temp[(size/2)-1]+Temp[size/2])/2;
	if((size%2)==0) return(m);
	else		return(Temp[size/2]);
}

int HIPL_Thinning(THEADER *it,THEADER *ot)
{
int i,j,flag=0,p2,p4,p6,p8,sum,s,N,m,max,n,l,k,prod1,prod2,r1,r2,c1,c2,p,prod3;
THEADER *temp;
DATA *ID;
temp=(THEADER *)malloc(sizeof(THEADER));
ID=(DATA *)malloc(sizeof(DATA));
ID->ROWS=it->img.ROWS;
ID->COLS=it->img.COLS;
ID->ver=it->ver;
i=HIPL_Initialise(temp,NULL,1,0,ID);
if(i<0) return -1;
i=HIPL_AllocateMem(temp);
if(i<0) return -1;
temp->MROWS=ot->MROWS=it->MROWS=3;
temp->MCOLS=ot->MCOLS=it->MCOLS=3;
temp->MORGX=ot->MORGX=it->MORGX=1;
temp->MORGY=ot->MORGY=it->MORGY=1;
	HIPL_Refresh(it,0);
	HIPL_Refresh(ot,0);
	do {
	max=HIPL_Getpart(it);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
	for(i=0;i<max;i++)
		for(j=0;j<it->img.COLS;j++)
			ot->iarr[i][j]=it->iarr[i][j];	
	i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
	} while(it->P.nleft);

do {
flag=0;
HIPL_Refresh(ot,0);
HIPL_Refresh(temp,0);
	do {
	max=HIPL_Getpart(ot);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
	for(i=0;i<max;i++)
	{
		for(j=0;j<ot->img.COLS;j++)
		{
			sum=0;
			m=-ot->MORGX;
			n=-ot->MORGY;
			if((i<ot->MORGX)||((max-i)<(ot->MROWS-ot->MORGX))||(j<ot->MORGY)||((ot->img.COLS-j)<(ot->MCOLS-ot->MORGY)))
					temp->iarr[i][j]=ot->iarr[i][j];
			else
			{
			prod3=1;
				for(k=0;k<ot->MROWS;k++)
				{
					for(l=0;l<ot->MCOLS;l++)
					{
						if((it->arrtype==0)&&((l!=1)||(k!=1)))
							prod3*=ot->iarr[i+n][j+m]/255;
						m++;
					}
					m=-ot->MORGY;
					n++;
				}
			m=-ot->MORGX;
			n=-ot->MORGY;
			
			N=0;
			if((ot->iarr[i][j]==255)&&(prod3==0))
			{
				for(k=0;k<ot->MROWS;k++)
				{
					for(l=0;l<ot->MCOLS;l++)
					{
						if((it->arrtype==0)&&((l!=1)||(k!=1)))
							N+=ot->iarr[i+n][j+m]/255;
						m++;
					}
					m=-ot->MORGY;
					n++;
				}
				if((N>1)&&(N<7))
				{	
					r1=-1;c1=0;s=0;	
					for(p=0;p<8;p++)
					{	
					switch(r1)
					{
					case -1:{
						switch(c1)
						{
						case -1:{
							r2=-1;
							c2=0;
							break;
							}
						case 0:{
							p2=ot->iarr[i+r1][j+c1];
							r2=-1;
							c2=1;
							break;
							}
						case 1:{
							r2=0;
							c2=1;
							break;
							}
						}
						break;
						}
					case 0:{
						switch(c1)
						{
						case -1:{
							p8=ot->iarr[i+r1][j+c1];
							r2=-1;
							c2=-1;
							break;
							}
						case 1:{
							p4=ot->iarr[i+r1][j+c1];
							r2=1;
							c2=1;
							break;
							}
						}
						break;
						}
					case 1:{
						switch(c1)
						{
						case -1:{
							r2=0;
							c2=-1;
							break;
							}
						case 0:{
							p6=ot->iarr[i+r1][j+c1];
							r2=1;
							c2=-1;
							break;
							}
						case 1:{
							r2=1;
							c2=0;
							break;
							}
						}
						}
					}
					if((ot->iarr[i+r1][j+c1]==0)&&(ot->iarr[i+r2][j+c2]==255)) 
					s++;
					r1=r2;c1=c2;
					}
					if(s==1)
					{
					prod1=p2*p4*p6;
					prod2=p4*p6*p8;
						if((prod1==0)&&(prod2==0))
						{
						flag=1;
						temp->iarr[i][j]=0;
						}
						else temp->iarr[i][j]=255;
					}
					else temp->iarr[i][j]=255;
					
					}
				
					else temp->iarr[i][j]=255;

				}				
				else temp->iarr[i][j] = ot->iarr[i][j];


			}
		}	
	}
	i=HIPL_Putpart(temp);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
	} while(temp->P.nleft);

	HIPL_Refresh(ot,0);
	HIPL_Refresh(temp,0);
				
	do {
	max=HIPL_Getpart(temp);
		if(max<0)
		{
			HIPL_Error("Error in Getpart");
			return(max);
		}
	for(i=0;i<max;i++)
	{
		for(j=0;j<temp->img.COLS;j++)
		{
			sum=0;
			m=-temp->MORGX;
			n=-temp->MORGY;
			if((i<temp->MORGX)||((max-i)<(temp->MROWS-temp->MORGX))||(j<temp->MORGY)||((temp->img.COLS-j)<(temp->MCOLS-temp->MORGY)))
					ot->iarr[i][j]=temp->iarr[i][j];
			else
			{
				prod3=1;
				for(k=0;k<temp->MROWS;k++)
				{
					for(l=0;l<temp->MCOLS;l++)
					{
						if((temp->arrtype==0)&&((l!=1)||(k!=1)))
							prod3*=temp->iarr[i+n][j+m]/255;
						m++;
					}
					m=-temp->MORGY;
					n++;
				}
			m=-temp->MORGX;
			n=-temp->MORGY;
			N=0;
			if((temp->iarr[i][j]==255)&&(prod3==0))
			{
				for(k=0;k<temp->MROWS;k++)
				{
					for(l=0;l<temp->MCOLS;l++)
					{
						if((temp->arrtype==0)&&((l!=1)||(k!=1)))
							N+=temp->iarr[i+n][j+m]/255;
						m++;
					}
					m=-temp->MORGY;
					n++;
				}
				if((N>1)&&(N<7))
				{	
					r1=-1;c1=0;s=0;	
					for(p=0;p<8;p++)
					{	
					switch(r1)
					{
					case -1:{
						switch(c1)
						{
						case -1:{
							r2=-1;
							c2=0;
							break;
							}
						case 0:{
							p2=temp->iarr[i+r1][j+c1];
							r2=-1;
							c2=1;
							break;
							}
						case 1:{
							r2=0;
							c2=1;
							break;
							}
						}
						break;
						}
					case 0:{
						switch(c1)
						{
						case -1:{
							p8=temp->iarr[i+r1][j+c1];
							r2=-1;
							c2=-1;
							break;
							}
						case 1:{
							p4=temp->iarr[i+r1][j+c1];
							r2=1;
							c2=1;
							break;
							}
						}
						break;
						}
					case 1:{
						switch(c1)
						{
						case -1:{
							r2=0;
							c2=-1;
							break;
							}
						case 0:{
							p6=temp->iarr[i+r1][j+c1];
							r2=1;
							c2=-1;
							break;
							}
						case 1:{
							r2=1;
							c2=0;
							break;
							}
						}
						break;
						}
					}
					if((temp->iarr[i+r1][j+c1]==0)&&(temp->iarr[i+r2][j+c2]==255)) s++;
					r1=r2;c1=c2;
					}
					if(s==1)
					{
					prod1=p2*p4*p8;
					prod2=p2*p6*p8;
						if((prod1==0)&&(prod2==0))
						{
						flag=1;
						ot->iarr[i][j]=0;
						}
						else ot->iarr[i][j]=255;
					}
					else ot->iarr[i][j]=255;
					
					}
					else ot->iarr[i][j]=255;

				}				
				else ot->iarr[i][j]=temp->iarr[i][j];


			}
		}	
	}
	i=HIPL_Putpart(ot);
		if(i<0)
		{
			HIPL_Error("Error in Putpart");
			return(i);
		}
	} while(ot->P.nleft);

} while(flag);
HIPL_FreeMem(temp);
HIPL_Closeall(temp);
free((char *)temp);
free(ID);
return 0;
}	
