#include "incl.h"
extern int HIPL_AllocateMem(THEADER *T);
extern int HIPL_FreeMem(THEADER *T);
extern int HIPL_Initialise(THEADER *T,char *name,short io,short arr_type,DATA *ID);
extern int HIPL_Getpart(THEADER *T);
extern int HIPL_Putpart(THEADER *T);
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_Refresh(THEADER *T,int flg);
extern int HIPL_Transpose(THEADER *it,THEADER *ot);
extern int HIPL_ERRNO;
extern void HIPL_Error(char *msg);
void bit_reversal(unsigned int *, int, int);
void WTS(float *, float *, int, int);
void FFT(float *, float *, float *,float *, int, int);


int HIPL_FFT(THEADER *it,THEADER *or,THEADER *oi)
{
	THEADER *tr;
	THEADER *ti;
	DATA *ID;
	float m1;
	int i,j,k,N,m,n2,l,max,c;
	unsigned int *L;
	float *wr,*wi;
	if(it->img.ROWS!=it->img.COLS){
		HIPL_ERRNO = -30;
		HIPL_Error("Error in FFT");
		return(-1);
	}
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=it->img.ROWS;
	ID->COLS=it->img.COLS;
	ID->ver=it->ver;
	N = it->img.ROWS;
	l = 1;
	m1=log10((double)N)/log10((double)2.0);
	m=(int)m1;
	for(i=0;i<m;i++) l<<=1;
	if(l != N){
		HIPL_ERRNO = -31; 
		HIPL_Error("Error in FFT");
		return(-1);
		}
	L=(unsigned int *)malloc(N*sizeof(unsigned int));
	bit_reversal(L,m,N);
	n2 = (N>>1) - 1;
	wr = (float *)malloc(n2*sizeof(float));
	wi = (float *)malloc(n2*sizeof(float));
	WTS(wr,wi,N,-1);
	c=0;
	do {
		max = HIPL_Getpart(it);
		for(i=0;i<max;i++){
			for(j=0;j<it->img.COLS;j++){
				k=L[j];
				or->farr[i][k] = (float) it->iarr[i][j];
				if(((j+i+c)%2) != 0) 
					or->farr[i][k] = -or->farr[i][k];
				oi->farr[i][k] = 0;
				}
			FFT(or->farr[i],oi->farr[i],wr,wi,m,N);
			}
		i=HIPL_Putpart(or);
		i=HIPL_Putpart(oi);
		c+=max;
	}while(it->P.nleft);
	HIPL_Refresh(or,0);
	HIPL_Refresh(oi,0);
	tr =(THEADER *)malloc(sizeof(THEADER));
	ti =(THEADER *)malloc(sizeof(THEADER));
	i=HIPL_Initialise(tr,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i=HIPL_AllocateMem(tr);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i=HIPL_Initialise(ti,NULL,1,1,ID);
	if(i<0){
	HIPL_Error("Error in initialisation");
	return -1;
	}  
	i=HIPL_AllocateMem(ti);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i = HIPL_Transpose(or,tr);
	if(i<0) return -1;
	i = HIPL_Transpose(oi,ti);
	if(i<0) return -1;
	HIPL_Refresh(tr,0);
	HIPL_Refresh(ti,0);
	HIPL_Refresh(or,0);
	HIPL_Refresh(oi,0);
	
	do {
		max  = HIPL_Getpart(tr);
		if(max<0){printf("error in getpart");return -1;}
		max  = HIPL_Getpart(ti);
		if(max<0){printf("error in getpart");return -1;}
		for(i=0;i<max;i++){
			for(j=0;j<it->img.COLS;j++) {
				k=L[j];
				or->farr[i][k]=tr->farr[i][j];
				oi->farr[i][k]=ti->farr[i][j];
				}
			FFT(or->farr[i],oi->farr[i],wr,wi,m,N);
			}
		max=HIPL_Putpart(or);
		if(max<0){printf("error in putpart");return -1;}
		max=HIPL_Putpart(oi);
		if(max<0){printf("error in putpart");return -1;}
	}while(tr->P.nleft);
	HIPL_Closeall(tr);
	HIPL_FreeMem(tr);
	HIPL_Closeall(ti);
	HIPL_FreeMem(ti);
	free(ID);
	return(0);
}

int HIPL_IFFT(THEADER *ir,THEADER *ii,THEADER *ot)
{
	THEADER *tr;
	THEADER *ti;
	DATA *ID;
	float m1,val;
	int i,j,k,N,m,n2,l,max,c,nsq;
	unsigned int *L;
	float *wr,*wi;
	if(ir->img.ROWS!=ir->img.COLS){
		HIPL_ERRNO = -30;
		HIPL_Error("Error in IFFT");
		return(-1);
	}
	if(ii->img.ROWS!=ii->img.COLS){
		HIPL_ERRNO = -30;
		HIPL_Error("Error in IFFT");
		return(-1);
	}
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=ir->img.ROWS;
	ID->COLS=ir->img.COLS;
	ID->ver=ir->ver;
	N = ir->img.ROWS;
	l = 1;
	m1=log10((double)N)/log10((double)2.0);
	m=(int)m1;
	for(i=0;i<m;i++) l<<=1;
	if(l != N){
		HIPL_ERRNO = -31; 
		HIPL_Error("Error in FFT");
		return(-1);
		}
	L=(unsigned int *)malloc(N*sizeof(unsigned int));
	bit_reversal(L,m,N);
	n2 = (N>>1) - 1;
	wr = (float *)malloc(n2*sizeof(float));
	wi = (float *)malloc(n2*sizeof(float));
	WTS(wr,wi,N,1);
	tr =(THEADER *)malloc(sizeof(THEADER));
	ti =(THEADER *)malloc(sizeof(THEADER));
	i=HIPL_Initialise(tr,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i=HIPL_AllocateMem(tr);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i=HIPL_Initialise(ti,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	i=HIPL_AllocateMem(ti);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
	c=0;
	do {
		max = HIPL_Getpart(ir);
		max = HIPL_Getpart(ii);
		for(i=0;i<max;i++){
			for(j=0;j<ir->img.COLS;j++){
				k=L[j];
				tr->farr[i][k] = ir->farr[i][j];
				ti->farr[i][k] = ii->farr[i][j];
				}
			FFT(tr->farr[i],ti->farr[i],wr,wi,m,N);
			}
		i=HIPL_Putpart(tr);
		i=HIPL_Putpart(ti);
	}while(tr->P.nleft);

	HIPL_Refresh(ii,0);
	HIPL_Refresh(ir,0);
	HIPL_Refresh(ti,0);
	HIPL_Refresh(tr,0);

	i = HIPL_Transpose(tr,ir);
	if(i<0) return -1;
	i = HIPL_Transpose(ti,ii);
	if(i<0) return -1;

	HIPL_Refresh(tr,0);
	HIPL_Refresh(ti,0);
	HIPL_Refresh(ir,0);
	HIPL_Refresh(ii,0);
	c=0;
	do {
		max  = HIPL_Getpart(ir);
		if(max<0){printf("error in getpart");return -1;}
		max  = HIPL_Getpart(ii);
		if(max<0){printf("error in getpart");return -1;}
		for(i=0;i<max;i++){
			for(j=0;j<ir->img.COLS;j++) {
				k=L[j];
				tr->farr[i][k]=ir->farr[i][j];
				ti->farr[i][k]=ii->farr[i][j];
				}
			FFT(tr->farr[i],ti->farr[i],wr,wi,m,N);
			}
		max=HIPL_Putpart(tr);
		if(max<0){printf("error in putpart");return -1;}
		max=HIPL_Putpart(ti);
		if(max<0){printf("error in putpart");return -1;}
	}while(tr->P.nleft);

	HIPL_Refresh(tr,0);
	HIPL_Refresh(ti,0);
	HIPL_Refresh(ir,0);
	HIPL_Refresh(ii,0);

	nsq=N*N;
	c=0;
	do{
	max=HIPL_Getpart(tr);
	for(i=0;i<max;i++)
		for(j=0;j<tr->img.COLS;j++)
			{
			val=tr->farr[i][j]/(float)nsq;
			if((i+j+c)%2!=0)
				ot->farr[i][j]=-val;
			else
				ot->farr[i][j]=val;
			}
	c+=max;
	max=HIPL_Putpart(ot);
	}while(ot->P.nleft);
	HIPL_Refresh(ot,0);
	HIPL_Closeall(tr);
	HIPL_FreeMem(tr);
	HIPL_Closeall(ti);
	HIPL_FreeMem(ti);
	free(ID);
	return(0);
}  
void bit_reversal(unsigned int *L, int m, int N)
{
	unsigned int MASK,C,A,j,k,i;
	for(k=0;k<N;k++)
		{
			MASK=1;
			C=0;
			for(i=0,j=m-1;i<m;i++,j--)
				{
					A=(k&MASK)>>i;
					A<<=j;
					C|=A;
					MASK=MASK<<1;
				}
			L[k] = C;
		}
}
void WTS(float *wr,float *wi, int N, int sign)
{
	int n2,i;
	float theta;
	n2 = (N>>1)-1;
	theta = 2.0 * (float)M_PI/((float)N);
	for(i=0;i<n2;i++)
		{
			wr[i]=(float)cos((double)(i+1) *theta);
			wi[i]=(float)sin((double)(i+1) *theta);
			if(sign==(int)(-1)) wi[i]=-wi[i];
		}
}

void FFT(float *xr, float *xi, float *wr, float *wi, int m, int N)
{
int ip,k,kk,l,incr,iter,j,i;
float tr,ti;
ip=1;
kk=(N>>1);
incr=2;

for(iter=0;iter<m;iter++)
{
	for(j=0;j<N;j+=incr)
	{
		i=j+ip;
		tr=xr[i];
		ti=xi[i];
		xr[i]=xr[j] - tr;
		xi[i]=xi[j] - ti;
		xr[j]=xr[j] + tr;
		xi[j]=xi[j] + ti;
	}
	if(iter!=0)
	{
		for(k=1;k<ip;k++)
		{
			l=k*kk-1;
			for(j=k;j<N;j+=incr)
			{
				i=j+ip;
				tr=xr[i]*wr[l]-xi[i]*wi[l];
				ti=xr[i]*wi[l]+xi[i]*wr[l];
				xr[i]=xr[j]-tr;
				xi[i]=xi[j]-ti;
				xr[j]=xr[j]+tr;
				xi[j]=xi[j]+ti;
			}
		}
	}
	kk>>=1;
	ip<<=1;
	incr<<=1;
	}
}

int HIPL_FFTLowhigh(THEADER *ir,THEADER *ii,THEADER *mr,THEADER *mi,double cf,short opr)
{

int M,i,j,row,r1,max;
double r2,h;
M=ir->img.ROWS>>1;
cf = ( cf/(double)M_PI)*(double)M;
cf *= cf;
row = 0;
HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);
HIPL_Refresh(mi,0);
HIPL_Refresh(mr,0);
do{
	max = HIPL_Getpart(ir);
	max = HIPL_Getpart(ii);

	for(i=0;i<max;i++){
		r1=(i+row-M)*(i+row-M);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((j-M)*(j-M)+r1);
			switch(opr){
				case 0:
					if(r2 < cf) 
					h = 1.0;
					else	  
					h=0;
					break;
				case 1:
					if(r2 > cf) 
					h=1.0;
					else	  
					h=0;
					break;
				case 2:
					h = 0.414*cf/(r2+0.414*cf);
					break;
	
				case 3:
					h=r2/(r2+0.414*cf);
					break;
				}
			mr->farr[i][j]=h*ir->farr[i][j];
			mi->farr[i][j]=h*ii->farr[i][j];
			
		}
	}
	row+=max;
	max=HIPL_Putpart(mr);
	max=HIPL_Putpart(mi);
   }while(mi->P.nleft);
return 0;
}


int HIPL_FFTBand(THEADER *ir,THEADER *ii,THEADER *mr,THEADER *mi,double cf1,double cf2,short opr)
{
int M,i,j,row,r1,max;
double r2,h;
M=ir->img.ROWS>>1;
cf1 = ( cf1/(double)M_PI)*(double)M;
cf1 *= cf1;
cf2 = ( cf2/(double)M_PI)*(double)M;
cf2 *= cf2;
row = 0;
HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);
HIPL_Refresh(mi,0);
HIPL_Refresh(mr,0);
do{
	max = HIPL_Getpart(ir);
	max = HIPL_Getpart(ii);

	for(i=0;i<max;i++){
		r1=(i+row-M)*(i+row-M);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((j-M)*(j-M)+r1);
			switch(opr){
				case 0:
					if((r2 < cf1) || (r2 > cf2))
					h = 0.0;
					else
					h=1.0;
					break;
				case 1:
					if((r2 < cf1) || (r2 > cf2))
					h=1.0;
					else	  
					h=0;
					break;
				}
			mr->farr[i][j]=h*ir->farr[i][j];
			mi->farr[i][j]=h*ii->farr[i][j];
			
		}
	}
	row+=max;
	max=HIPL_Putpart(mr);
	max=HIPL_Putpart(mi);
   }while(mi->P.nleft);
return 0;
}

int HIPL_FFTMasklh(float **mask,int isize,int msize,double cf,int win,int opr)
{
int M,i,j,row,r1,max,ii1,jj,n,m2,m3;
float win1,theta,nsq,nsqrt;
double r2,h;
DATA *ID;
THEADER *ir,*ii,*ot;
ID=(DATA *)malloc(sizeof(DATA));
ir = (THEADER *)malloc(sizeof(THEADER));
ii = (THEADER *)malloc(sizeof(THEADER));
ot = (THEADER *)malloc(sizeof(THEADER));
ID->COLS=isize;
ID->ROWS=isize;
ID->ver=0;

i = HIPL_Initialise(ir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ii,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ii);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ot,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ot);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

M=ir->img.ROWS>>1;
cf = ( cf/(double)M_PI)*(double)M;
cf *= cf;
row = 0;
do{
	for(i=0;i<isize;i++){
		r1=(M-i-row)*(M-i-row);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((M-j)*(M-j)+r1);
			switch(opr){
				case 0:
					if(r2 < cf) 
					h = 1.0;
					else	  
					h=0;
					break;
				case 1:
					if(r2 > cf) 
					h=1.0;
					else	  
					h=0;
					break;
				case 2:
					h = 0.414*cf/(r2+0.414*cf);
					break;
	
				case 3:
					h=r2/(r2+0.414*cf);
					break;
				}
			ir->farr[i][j]=(float)h;
			if((i+j)%2!=0)ir->farr[i][j] = -h;
			ii->farr[i][j]=0.0;
			
		}
	}
	row+=max;
	max=HIPL_Putpart(ir);
	max=HIPL_Putpart(ii);
   }while(ii->P.nleft);

HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);

HIPL_IFFT(ir,ii,ot);

/*HIPL_Refresh(ot,0);
do{
max=HIPL_Getpart(ot);
for(i =0 ;i<max;i++)
	for(j=0;j<ot->img.COLS;j++)
	{
		printf("\n %d %d %f ",i,j,ot->farr[i][j]);
	}
}while(ot->P.nleft);*/
HIPL_Refresh(ot,0);
n =( msize-1)/2;
m2 = M-n;
m3 = M+n;
do{
max=HIPL_Getpart(ot);
for(i = m2;i<=m3;i++)
	for(j=m2;j<=m3;j++)
	{
		mask[i-m2][j-m2]=ot->farr[i][j];
	}
}while(ot->P.nleft);
theta = (float)M_PI/((float)n*sqrt((double)2.0));
for(i=0;i<msize;i++){
	ii1 = (i-n)*(i-n);
	for(j=0;j<msize;j++){
		jj=(j-n)*(j-n);
		nsq=(float)(ii1+jj);
		nsqrt = sqrt((double)(nsq))/sqrt((double)2.0);
		
		switch(win){
		case 1:
		break;

		case 2:
		win1 = 0.5+0.5*cos((double)(theta*nsqrt));
		mask[i][j]*=win1;
		break;
		
		case 3:
		win1 = 0.54+0.46*cos((double)(theta*nsqrt));
		mask[i][j]*=win1;
		break;
		
		case 4:
		win1 = 0.42+0.5*cos((double)(theta*nsqrt))+0.08*cos((double)(2.0*theta*nsqrt));
		mask[i][j]*=win1;
		break;
		}
		}
	}
HIPL_Closeall(ii);
HIPL_FreeMem(ii);
HIPL_Closeall(ir);
HIPL_FreeMem(ir);
HIPL_Closeall(ot);
HIPL_FreeMem(ot);
free(ir);
free(ii);
free(ot);
free(ID);
return 0;
}

int HIPL_FFTMaskbp(float **mask,int isize,int msize,double cf1,double cf2,int win,int opr)
{
int M,i,j,row,r1,max,ii1,jj,n,m2,m3;
float win1,theta,nsq,nsqrt;
double r2,h;
DATA *ID;
THEADER *ir,*ii,*ot;
ID=(DATA *)malloc(sizeof(DATA));
ir = (THEADER *)malloc(sizeof(THEADER));
ii = (THEADER *)malloc(sizeof(THEADER));
ot = (THEADER *)malloc(sizeof(THEADER));
ID->COLS=isize;
ID->ROWS=isize;
ID->ver=0;

i = HIPL_Initialise(ir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ii,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ii);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ot,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ot);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

M=ir->img.ROWS>>1;
cf1 = ( cf1/(double)M_PI)*(double)M;
cf1 *= cf1;
cf2 = ( cf2/(double)M_PI)*(double)M;
cf2 *= cf2;
row = 0;
do{
	for(i=0;i<isize;i++){
		r1=(M-i-row)*(M-i-row);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((M-j)*(M-j)+r1);
			switch(opr){
				case 0:
					if((r2 > cf1)&&(r2<cf2)) 
					h = 1.0;
					else	  
					h=0;
					break;
				case 1:
					if((r2<cf1)||(r2 > cf2))
					h=1.0;
					else	  
					h=0;
					break;
				}
			ir->farr[i][j]=(float)h;
			if((i+j)%2!=0)ir->farr[i][j] = -h;
			ii->farr[i][j]=0.0;
			
		}
	}
	row+=max;
	max=HIPL_Putpart(ir);
	max=HIPL_Putpart(ii);
   }while(ii->P.nleft);

HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);

HIPL_IFFT(ir,ii,ot);

HIPL_Refresh(ot,0);
n =( msize-1)/2;
m2 = M-n;
m3 = M+n;
do{
max=HIPL_Getpart(ot);
for(i = m2;i<=m3;i++)
	for(j=m2;j<=m3;j++)
	{
		mask[i-m2][j-m2]=ot->farr[i][j];
	}
}while(ot->P.nleft);
theta = (float)M_PI/((float)n*sqrt((double)2.0));
for(i=0;i<msize;i++){
	ii1 = (i-n)*(i-n);
	for(j=0;j<msize;j++){
		jj=(j-n)*(j-n);
		nsq=(float)(ii1+jj);
		nsqrt = sqrt((double)(nsq))/sqrt((double)2.0);
		
		switch(win){
		case 1:
		break;

		case 2:
		win1 = 0.5+0.5*cos((double)(theta*nsqrt));
		mask[i][j]*=win1;
		break;
		
		case 3:
		win1 = 0.54+0.46*cos((double)(theta*nsqrt));
		mask[i][j]*=win1;
		break;
		
		case 4:
		win1 = 0.42+0.5*cos((double)(theta*nsqrt))+0.08*cos((double)(2.0*theta*nsqrt));
		mask[i][j]*=win1;
		break;
		}
		}
	}
HIPL_Closeall(ii);
HIPL_FreeMem(ii);
HIPL_Closeall(ir);
HIPL_FreeMem(ir);
HIPL_Closeall(ot);
HIPL_FreeMem(ot);
free(ir);
free(ii);
free(ot);
free(ID);
return 0;
}

int HIPL_IIRMasklh(float **a,float **b,int isize,int order,double cf,int win,int opr)
{
int M,i,j,row,r1,max,ii1,jj,n,m2,m1,n1,n2,kk1,kk2,offset,k,l,m,msq;
float win1,theta,nsq,nsqrt,sum;
float *h1,*A;
double r2,h;
DATA *ID;
THEADER *ir,*ii,*ot,*iir;

ID=(DATA *)malloc(sizeof(DATA));
ir = (THEADER *)malloc(sizeof(THEADER));
ii = (THEADER *)malloc(sizeof(THEADER));
ot = (THEADER *)malloc(sizeof(THEADER));
iir = (THEADER *)malloc(sizeof(THEADER));

ID->COLS=isize;
ID->ROWS=isize;
ID->ver=0;

i = HIPL_Initialise(ir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ii,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ii);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ot,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ot);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

//procedure for finding impulse response

M=ir->img.ROWS>>1;
cf = ( cf/(double)M_PI)*(double)M;
cf *= cf;
row = 0;
do{
	for(i=0;i<isize;i++){
		r1=(M-i-row)*(M-i-row);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((M-j)*(M-j)+r1);
			switch(opr){
				case 0:
					if(r2 < cf) 
					h = 1.0;
					else	  
					h=0;
					break;
				case 1:
					if(r2 > cf) 
					h=1.0;
					else	  
					h=0;
					break;
				case 2:
					h = 0.414*cf/(r2+0.414*cf);
					break;
	
				case 3:
					h=r2/(r2+0.414*cf);
					break;
				}
			ir->farr[i][j]=(float)h;
			if((i+j)%2!=0)ir->farr[i][j] = -h;
			ii->farr[i][j]=0.0;
			
		}
	}
	row+=max;
	max=HIPL_Putpart(ir);
	max=HIPL_Putpart(ii);
   }while(ii->P.nleft);

HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);

HIPL_IFFT(ir,ii,ot);

HIPL_Closeall(ii);
HIPL_FreeMem(ii);
HIPL_Closeall(ir);
HIPL_FreeMem(ir);

free(ir);
free(ii);

/*HIPL_Refresh(ot,0);
do{
max=HIPL_Getpart(ot);
for(i =0 ;i<max;i++)
	for(j=0;j<ot->img.COLS;j++)
	{
		printf("\n %d %d %f ",i,j,ot->farr[i][j]);
	}
}while(ot->P.nleft);
fflush(stdout);*/
HIPL_Refresh(ot,0);

i = HIPL_Initialise(iir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(iir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

//multiplying with windowing function

n=ot->img.COLS;
n1 = n/2;
theta = (float)M_PI/((float)n1*sqrt((double)2.0));
do{
max = HIPL_Getpart(ot);
for(i=0;i<n;i++){
	ii1 = (i-n1)*(i-n1);
	for(j=0;j<n;j++){
		jj=(j-n1)*(j-n1);
		nsq=(float)(ii1+jj);
		nsqrt = sqrt((double)(nsq))/sqrt((double)2.0);
		
		switch(win){
		case 1:
		iir->farr[i][j] = ot->farr[i][j];
		break;

		case 2:
		win1 = 0.5+0.5*cos((double)(theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		
		case 3:
		win1 = 0.54+0.46*cos((double)(theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		
		case 4:
		win1 = 0.42+0.5*cos((double)(theta*nsqrt))+0.08*cos((double)(2.0*theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		}
		}
	}
HIPL_Putpart(iir);
}while(iir->P.nleft);

HIPL_Closeall(ot);
HIPL_FreeMem(ot);
free(ot);
//procedure for finding iir mask 
n=iir->img.ROWS;
offset = order - 1;
m1 = n/2 + offset;
m2 = n/2 - offset;
msq = m1 * m1;
h1 = (float *)malloc(msq*sizeof(float));
n1 = (order+1)*(order+1) - 1;
n2 = (order+1)*(order+1);
A = (float *)malloc(n1*n2*sizeof(float));
fflush(stdout);

HIPL_Refresh(iir,0);

do{
max=HIPL_Getpart(iir);
for(i = m2;i<n;i++)
	for(j=m2;j<n;j++)
	{
		h1[(i-m2)*m1 +j-m2] = iir->farr[i][j];
	}
}while(iir->P.nleft);

HIPL_Refresh(iir,0);
kk1 = 0;
for(l=0;l<=order;l++)
	for(k=0;k<=order;k++)
		{
		if((l+k) == 0)continue;
		kk2 = 0;
	for(i=0;i<=order;i++)
		for(j=0;j<=order;j++)
		{
		if((i+j) == 0) continue;
		if(kk2>=kk1)
			{
			sum = (float)0.0;
			for(m=0;m<m1;m++)
				for(n=0;n<m1;n++)
				{
				if((n<=order)&&(m<=order)) continue;
				if(((m-i)<0)||((n-j)<0)||((m-l)<0)||((n-k)<0)) continue;
				sum += h1[(m-i)*m1+(n-j)]*h1[(m-l)*m1+(n-k)];
				}
			A[kk1*n2+kk2] = sum;
			}
		else 
		A[kk1*n2+kk2] = A[kk2*n2+kk1];
		kk2++;
		}
sum=(float)0.0;
for(m=0;m<m1;m++)
	for(n=0;n<m1;n++)
	{
		if((n<=order) && (m<=order)) continue;
		if(((m-l)<0)||((n-k)<0)) continue;
		sum-=h1[m*m1+n]*h1[(m-l)*m1+(n-k)];
	}
A[kk1*n2+n1] = sum;
kk1++;
}
SIMQ(A,n1);
b[0][0] = 1.0;
kk1 = 0;
for(i = 0;i<=order;i++)
	for(j = 0;j<=order;j++)
	{
	if((i+j)==0) continue;
	b[i][j]=A[kk1*n2+n1];
	kk1++;
	}
for(m=0;m<=order;m++)
	for(n=0;n<=order;n++)
		{
		a[m][n] = (float)0.0;
		for(i=0;i<=order;i++)
			for(j=0;j<=order;j++)
			{
				if(((m-i)<0)||((n-j)<0)) continue;
				a[m][n]+=b[i][j]*h1[(m-i)*m1+(n-j)];
			}
		}
free(ID);
free(A);
free(h1);
HIPL_Closeall(iir);
HIPL_FreeMem(iir);
free(iir);
return 0;
}

int HIPL_IIRMaskbp(float **a,float **b,int isize,int order,double cf1,double cf2,int win,int opr)
{
int M,i,j,row,r1,max,ii1,jj,n,m2,m1,n1,n2,kk1,kk2,offset,k,l,m,msq;
float win1,theta,nsq,nsqrt,sum;
float *h1,*A;
double r2,h;
DATA *ID;
THEADER *ir,*ii,*ot,*iir;

ID=(DATA *)malloc(sizeof(DATA));
ir = (THEADER *)malloc(sizeof(THEADER));
ii = (THEADER *)malloc(sizeof(THEADER));
ot = (THEADER *)malloc(sizeof(THEADER));
iir = (THEADER *)malloc(sizeof(THEADER));

ID->COLS=isize;
ID->ROWS=isize;
ID->ver=0;

i = HIPL_Initialise(ir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ii,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ii);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_Initialise(ot,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(ot);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

//procedure for finding impulse response

M=ir->img.ROWS>>1;
cf1 = ( cf1/(double)M_PI)*(double)M;
cf1 *= cf1;
cf2 = ( cf2/(double)M_PI)*(double)M;
cf2 *= cf2;
row = 0;
do{
	for(i=0;i<isize;i++){
		r1=(M-i-row)*(M-i-row);
		for(j=0;j<ii->img.COLS;j++){
			r2=(double)((M-j)*(M-j)+r1);
			switch(opr){
				case 0:
					if((r2 > cf1)&&(r2 < cf2)) 
					h = 1.0;
					else	  
					h=0;
					break;
				case 1:
					if((r2 < cf1)||(r2 > cf2)) 
					h=1.0;
					else	  
					h=0;
					break;
				}
			ir->farr[i][j]=(float)h;
			if((i+j)%2!=0)ir->farr[i][j] = -h;
			ii->farr[i][j]=0.0;
			
		}
	}
	row+=max;
	max=HIPL_Putpart(ir);
	max=HIPL_Putpart(ii);
   }while(ii->P.nleft);

HIPL_Refresh(ir,0);
HIPL_Refresh(ii,0);

HIPL_IFFT(ir,ii,ot);

HIPL_Closeall(ii);
HIPL_FreeMem(ii);
HIPL_Closeall(ir);
HIPL_FreeMem(ir);

free(ir);
free(ii);

/*HIPL_Refresh(ot,0);
do{
max=HIPL_Getpart(ot);
for(i =0 ;i<max;i++)
	for(j=0;j<ot->img.COLS;j++)
	{
		printf("\n %d %d %f ",i,j,ot->farr[i][j]);
	}
}while(ot->P.nleft);
fflush(stdout);*/
HIPL_Refresh(ot,0);

i = HIPL_Initialise(iir,NULL,1,1,ID);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  
i = HIPL_AllocateMem(iir);
	if(i<0){
		HIPL_Error("Error in initialisation");
		return -1;
		}  

//multiplying with windowing function

n=ot->img.COLS;
n1 = n/2;
theta = (float)M_PI/((float)n1*sqrt((double)2.0));
do{
max = HIPL_Getpart(ot);
for(i=0;i<n;i++){
	ii1 = (i-n1)*(i-n1);
	for(j=0;j<n;j++){
		jj=(j-n1)*(j-n1);
		nsq=(float)(ii1+jj);
		nsqrt = sqrt((double)(nsq))/sqrt((double)2.0);
		
		switch(win){
		case 1:
		iir->farr[i][j] = ot->farr[i][j];
		break;

		case 2:
		win1 = 0.5+0.5*cos((double)(theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		
		case 3:
		win1 = 0.54+0.46*cos((double)(theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		
		case 4:
		win1 = 0.42+0.5*cos((double)(theta*nsqrt))+0.08*cos((double)(2.0*theta*nsqrt));
		iir->farr[i][j] = win1*ot->farr[i][j];
		break;
		}
		}
	}
HIPL_Putpart(iir);
}while(iir->P.nleft);

HIPL_Closeall(ot);
HIPL_FreeMem(ot);
free(ot);
//procedure for finding iir mask 
n=iir->img.ROWS;
offset = order - 1;
m1 = n/2 + offset;
m2 = n/2 - offset;
msq = m1 * m1;
h1 = (float *)malloc(msq*sizeof(float));
n1 = (order+1)*(order+1) - 1;
n2 = (order+1)*(order+1);
A = (float *)malloc(n1*n2*sizeof(float));
fflush(stdout);

HIPL_Refresh(iir,0);

do{
max=HIPL_Getpart(iir);
for(i = m2;i<n;i++)
	for(j=m2;j<n;j++)
	{
		h1[(i-m2)*m1 +j-m2] = iir->farr[i][j];
	}
}while(iir->P.nleft);

HIPL_Refresh(iir,0);
kk1 = 0;
for(l=0;l<=order;l++)
	for(k=0;k<=order;k++)
		{
		if((l+k) == 0)continue;
		kk2 = 0;
	for(i=0;i<=order;i++)
		for(j=0;j<=order;j++)
		{
		if((i+j) == 0) continue;
		if(kk2>=kk1)
			{
			sum = (float)0.0;
			for(m=0;m<m1;m++)
				for(n=0;n<m1;n++)
				{
				if((n<=order)&&(m<=order)) continue;
				if(((m-i)<0)||((n-j)<0)||((m-l)<0)||((n-k)<0)) continue;
				sum += h1[(m-i)*m1+(n-j)]*h1[(m-l)*m1+(n-k)];
				}
			A[kk1*n2+kk2] = sum;
			}
		else 
		A[kk1*n2+kk2] = A[kk2*n2+kk1];
		kk2++;
		}
sum=(float)0.0;
for(m=0;m<m1;m++)
	for(n=0;n<m1;n++)
	{
		if((n<=order) && (m<=order)) continue;
		if(((m-l)<0)||((n-k)<0)) continue;
		sum-=h1[m*m1+n]*h1[(m-l)*m1+(n-k)];
	}
A[kk1*n2+n1] = sum;
kk1++;
}
SIMQ(A,n1);
b[0][0] = 1.0;
kk1 = 0;
for(i = 0;i<=order;i++)
	for(j = 0;j<=order;j++)
	{
	if((i+j)==0) continue;
	b[i][j]=A[kk1*n2+n1];
	kk1++;
	}
for(m=0;m<=order;m++)
	for(n=0;n<=order;n++)
		{
		a[m][n] = (float)0.0;
		for(i=0;i<=order;i++)
			for(j=0;j<=order;j++)
			{
				if(((m-i)<0)||((n-j)<0)) continue;
				a[m][n]+=b[i][j]*h1[(m-i)*m1+(n-j)];
			}
		}
free(ID);
free(A);
free(h1);
HIPL_Closeall(iir);
HIPL_FreeMem(iir);
free(iir);
return 0;
}

int SIMQ(float *a, int N)
{
	int i,j,l,k,N1;
	float big,temp;

	N1 = N+1;
	for(j=0;j<N;j++)
	{
		big=(float)fabs((double)a[j*N1+j]);
		l=j;
		for(k=j;k<N;k++)
			{
			if(big<(float)fabs((double)a[k*N1+j]))
				{
				big = (float)fabs((double)a[k*N1+j]);
				l=k;
				}
			}
		if(big<1.e-10)
		{
			printf("\n No unique solution");
			return -1;
		}
	for(k=0;k<N1;k++)
		{
		temp = a[j*N1+k];
		a[j*N1+k]=a[l*N1+k];
		a[l*N1+k]=temp;
		}
	for(k=j+1;k<N1;k++)
		a[j*N1+k]/=a[j*N1+j];
		a[j*N1+j]=1.0;
	
	for(i=0;i<N;i++)
	{
		if(i!=j)
			{
			for(k=j+1;k<N1;k++)
				a[i*N1+k]-=a[i*N1+j]*a[j*N1+k];
			a[i*N1+j] = (float)0.0;
			}
	}
	}
return 0;
}



