#include "incl.h"

int  HIPL_ReadImagePGM(FILE *,THEADER *t);
int  HIPL_ReadImagePPM(FILE *fp1,THEADER *t);
int  HIPL_GetImageHeaderPGM(FILE *, THEADER *t);
int  HIPL_GetImageTypePGM(THEADER *t);
int  HIPL_GetImageRowsPGM(THEADER *t);
int  HIPL_GetImageColsPGM(THEADER *t);
int  HIPL_PutImageHeaderPGM(FILE *fp,THEADER *T);
int  HIPL_GetImageHeaderPPM(FILE *fp, THEADER *t);
int  HIPL_WriteImagePPM(FILE *fp, THEADER *t);

extern int HIPL_ERRNO;

/*******************************************************************
 *	FUNCTION: ReadImage
 *	PARAMETERS: 
 *		FILE *fp     - Pointer to file containing image data(I)
 *              THEADER  *t  -  contains header information(O)
 *	RETURN VALUES: 0 upon success
 *                 -1 upon failure			
 ******************************************************************/

int HIPL_ReadImagePGM(FILE *fp1,THEADER *t)

{
	int temp;
	int data, i, j;
	int numrows, numcols;
	long int NUM;

	if ((data = HIPL_GetImageHeaderPGM(fp1, t)) != 0) {
		HIPL_ERRNO=-11;
		return(-1);
	}
	if ((data = HIPL_GetImageTypePGM(t)) == 50) {
		HIPL_ERRNO=-10;
		return(-1);
	}

	if ((numrows = HIPL_GetImageRowsPGM(t)) <= 0) {
		HIPL_ERRNO=-12;
		return(1);
	}

	if ((numcols = HIPL_GetImageColsPGM(t)) <= 0) {
		HIPL_ERRNO=-13;
		return(1);
	};

	numrows=t->img.ROWS;
	numcols=t->img.COLS;
	
	NUM=(numrows)*(numcols);
		for(i=0;i<numrows;i++)
		{
			for (j=0;j<numcols; j++) 
			{
				temp=0;

				if(t->ver==0)
				{
					temp = fgetc(fp1);

					if (ferror(fp1)==1)
					{
						HIPL_ERRNO=-14;
						clearerr(fp1);
						return(-1);
					}

					if (feof(fp1)==1)
					{
						HIPL_ERRNO=-14;
						clearerr(fp1);
						return(-1);
					}
				}

				if(t->ver==1)
				{
					fscanf(fp1,"%d",&temp);

					if (ferror(fp1)==1)
					{
						HIPL_ERRNO=-14;
						clearerr(fp1);
						return(-1);
					}

					if (feof(fp1)==1)
					{
						HIPL_ERRNO=-14;
						clearerr(fp1);
						return(-1);
					}
				}

			
			if(t->ver==0)
			{
				fputc(temp,t->fptr);

				if (ferror(t->fptr)==1)
				{
					HIPL_ERRNO=-14;
					clearerr(t->fptr);
					return(-1);
				}
			}

			if(t->ver==1)
			{
				fprintf(t->fptr,"%3d ",temp);

				if (ferror(t->fptr)==1)
				{
					HIPL_ERRNO=-14;
					clearerr(t->fptr);
					return(-1);
				}
			}
			
			/*if((temp<0)||(temp>255))
			{
				fprintf(stdout,"ROW = %d   COL = %d ",i,j);
				Read1char();
				return(-1);
			}*/
		}

	}
	return(0);

}


/************************************************************************
 *									*
 *	FUNCTION: HIPL_GetImageHeader					*
 *									*
 *	PARAMETERS:							*
 *		FILE *fp       - File containing image data (I)		*
 *              THEADER t - Data structure containing header info (O)	*
 *									*
 *	RETURN VALUES: 0 upon success					*
 *                     1 upon failure					*
 *									*
 ************************************************************************/

int HIPL_GetImageHeaderPGM(FILE *fp, THEADER *t)
{
	int tmp,i,rowcount=1, AllReadFlag=0;
	char tmpline[81];


	t->img.TYPE = 50;
	t->img.ROWS = t->img.COLS = 0;

	/* If the first two bytes are "P5" or "P2" then the image is PGM */
	if (((tmp = fgetc(fp)) != 0) && (tmp != 'P')) {
	     ungetc(tmp, fp);
	     t->img.TYPE = 20;
	}
	if (tmp == 'P') {
	     if (((tmp = fgetc(fp)) == '5') || (tmp=='2'))
	     {
		  t->img.TYPE = 10;
		  if(tmp=='2')
		       t->ver=1;
		  if(tmp=='5')
		       t->ver=0;
	     }
	     else
		  t->img.TYPE = 50;
	}
	

	if (tmp == 0) {
	     HIPL_ERRNO=-11;
	     return(-1);
	}
	/* If this is a PGM image, then process the header */
	if (t->img.TYPE == 10) {
	     tmp = fgetc(fp);                    /* Discard a \n */
	     while (1) {
		  if (fgets(tmpline, 80, fp) != NULL) {
		       if (tmpline[0] == '#')   /* Comment. Ignore it! */
			    rowcount++ ;
		       else {
			    if(AllReadFlag==1) 
				 break;
			    if (t->img.ROWS== 0)
				 sscanf(tmpline, "%d %d", &t->img.COLS,
					&t->img.ROWS);
			    else {
				 sscanf(tmpline,"%d",&t->img.levels);
				 AllReadFlag=1;
			    }
			    rowcount++;
		       }
		  }
	     }
	} else if (t->img.TYPE == 20) {
	     /* If RAW images, prompt user for image dimensions */
	     fprintf(stderr, "Image Type is RAW\n");
		while (AllReadFlag != 1) {
		     fprintf(stderr, "Input Number of Rows in the Image: ");
		     fflush(stderr);
		     fgets(tmpline, 80, stdin);
		     if (sscanf(tmpline, "%d", &(t->img.ROWS)) != 1)
			  AllReadFlag = 0;
		     else
			  AllReadFlag = 1;
		}
		AllReadFlag = 0;
		while (AllReadFlag != 1) {
		     fprintf(stderr, "Input Number of Columns in the Image: ");
		     fflush(stderr);
		     fgets(tmpline, 80, stdin);
		     if (sscanf(tmpline, "%d", &(t->img.COLS)) != 1)
			  AllReadFlag = 0;
		     else
			  AllReadFlag = 1;
		}
	}
	fseek(fp,0,0);
	for(i=0;i<rowcount;i++)
	     fgets(tmpline,80,fp);
	
	return(0);
}

int HIPL_GetImageTypePGM(THEADER *t)
{
	return(t->img.TYPE);
}

int HIPL_GetImageRowsPGM(THEADER *t)
{
	return(t->img.ROWS);
}

int HIPL_GetImageColsPGM(THEADER *t)
{
	return(t->img.COLS);
}


/********************************************************************
 *	FUNCTION: HIPL_WriteImagePGM
 *	PARAMETERS:
 *		FILE *fp         - File pointer for output file (I)
 *      THEADER t        - Header information (I)
 *	RETURN VALUES: 0 upon success
 *                 -1 upon failure
 *******************************************************************/

int HIPL_WriteImagePGM(FILE *fp, THEADER *t)
{
	int i,j;
	int numrows, numcols;
	FILE *fd;
	int temp;
	long int NUM;
	NUM=(t->img.ROWS)*(t->img.COLS);

	/* if (HIPL_GetImageTypePGM(t) == 10) {
		if (HIPL_PutImageHeaderPGM(fp, t) != 0) {
			HIPL_ERRNO=-15;
			return(-1);
		}
	}
	if ((numrows = HIPL_GetImageRowsPGM(t)) <= 0) {
		HIPL_ERRNO=-16;
		return(-1);
	}
	if ((numcols = HIPL_GetImageColsPGM(t)) <= 0) {
		HIPL_ERRNO=-16;
		return(-1);
		} */

	if (t->ver == 0)
	     fprintf(fp, "P5\n%d %d\n%d\n", t->img.COLS, t->img.ROWS,
		     255);
	else
	     fprintf(fp, "P2\n%d %d\n%d\n", t->img.COLS, t->img.ROWS,
		     255);
	
	if(t->memflag!=1)
	{

		if(ISUNIX)
			fd=fopen(t->tempfname,"r");
		else
		{
			if(t->ver==0)
				fd=fopen(t->tempfname,"rb");
			else
				fd=fopen(t->tempfname,"r");
		}
		if(fd==NULL)
		{
			HIPL_ERRNO=-7;
			return(-1);
		}

		for(i=0;i<NUM;i++){
			if(t->ver==0)
			{
				temp=fgetc(fd);
				fputc(temp,fp);
			}
			if(t->ver==1)
			{
				fscanf(fd,"%d",&temp);
				if(ferror(fd)==1)
				{
					HIPL_ERRNO=-18;
					clearerr(fd);
					return(-1);
				}

				fprintf(fp,"%d ",temp);
				if(ferror(fp)==1)
				{
					HIPL_ERRNO=-18;
					clearerr(fp);
					return(-1);
				}

			}
		}
		fclose(fd);
	}

	if(t->memflag==1)
	{
		for(i=0;i<t->img.ROWS;i++)
		{
			for(j=0;j<t->img.COLS;j++)
			{
				if(t->ver==0)
					fputc(t->iarr[i][j],fp);

				if(t->ver==1)
					fprintf(fp,"%d ",t->iarr[i][j]);
					if(ferror(fp)==1)
					{
						HIPL_ERRNO=-18;
						clearerr(fp);
						return(-1);
					}

			}
		}
	}
	return(0);
}

int HIPL_PutImageHeaderPGM(FILE *fp, THEADER *t)
{
	if (HIPL_GetImageTypePGM(t) == 10) {
		if(t->ver==1)
		fprintf(fp, "P2\n#CREATED BY IPTOOL 98\n%d %d\n255\n",  t->img.COLS, t->img.ROWS);
		if(t->ver==0)
		fprintf(fp, "P5\n#CREATED BY IPTOOL 98\n%d %d\n255\n", t->img.COLS, t->img.ROWS);

	}

	return(0);
}

/************************************************************************
 *									*
 *	FUNCTION: HIPL_GetImageHeaderPPM				*
 *									*
 *	PARAMETERS:							*
 *		FILE *fp       - File containing image data (I)		*
 *              THEADER t - Data structure containing header info (O)	*
 *									*
 *	RETURN VALUES: 0 upon success					*
 *                     1 upon failure					*
 *									*
 ************************************************************************/

int HIPL_GetImageHeaderPPM(FILE *fp, THEADER *t)
{
     int tmp,i,rowcount=1, AllReadFlag=0;
     char tmpline[81];
     
     
     t->img.TYPE = 50;
     t->img.ROWS = t->img.COLS = 0;
     DB_printf(("Entered GetImageHeaderPPM\n"));

     /* If the first two bytes are "P6" or "P3" then the image is PPM
     if (((tmp = fgetc(fp)) != 0) && (tmp != 'P')) {
	  ungetc(tmp, fp);
	  t->img.TYPE = 20;
     }
     if (tmp == 'P') {
	  if (((tmp = fgetc(fp)) == '6')||(tmp=='3'))
	  {
	       t->img.TYPE = PPM;
	       if(tmp=='3')
		    t->ver=1;
	       if(tmp=='6'))
	       t->ver=0;
     }
     else
     t->img.TYPE = 50;

     if (tmp == 0) {
	  HIPL_ERRNO=-11;
	  return(-1);
	  } */
     tmp = fgetc(fp); tmp = fgetc(fp);   /* Discard the magic number */
     if (tmp == '3')
	  t->ver = 1;
     else
	  t->ver = 0;
     t->img.TYPE = PPM;
     tmp = fgetc(fp);		/* Discard the \n */

     /* Now we really start reading the image header */
     while (1) {
	  if (fgets(tmpline, 80, fp) != NULL) {
	       DB_printf(("Reading Line: %s\n", tmpline));
	       if (tmpline[0] == '#')   /* Comment. Ignore it! */
		    rowcount++ ;
	       else {
		    if(AllReadFlag==1)
			 goto ENDHEADER;
		    if (t->img.ROWS== 0) {
			 sscanf(tmpline, "%d %d", &t->img.COLS,
				&t->img.ROWS);
			 DB_printf(("Rows: %d\tCols: %d\n", t->img.ROWS,
				    t->img.COLS));
		    } else {
			 sscanf(tmpline,"%d",&t->img.levels);
			 AllReadFlag=1;
		    }
		    rowcount++;
	       }
	  }
     }

     ENDHEADER: fseek(fp,0,0);
     for(i=0;i<rowcount;i++)
	  fgets(tmpline,80,fp);
     
     DB_printf(("Leaving GetImageHeaderPPM\n"));
     return(0);
}

/*******************************************************************
 *	FUNCTION: ReadImagePPM
 *	PARAMETERS: 
 *		FILE *fp     - Pointer to file containing image data(I)
 *              THEADER  *t  -  contains header information(O)
 *	RETURN VALUES: 0 upon success
 *                 -1 upon failure			
 ******************************************************************/

int HIPL_ReadImagePPM(FILE *fp1,THEADER *t)

{
	int temp[3];
	int data, i, j, k;
	int numrows, numcols;
	long int NUM;

	if ((data = HIPL_GetImageHeaderPPM(fp1, t)) != 0) {
		HIPL_ERRNO=-11;
		return(-1);
	}

	/* if ((data = HIPL_GetImageTypePGM(t)) == 50) {
		HIPL_ERRNO=-10;
		return(-1);
	}

	if ((numrows = HIPL_GetImageRowsPGM(t)) <= 0) {
		HIPL_ERRNO=-12;
		return(1);
	}

	if ((numcols = HIPL_GetImageColsPGM(t)) <= 0) {
		HIPL_ERRNO=-13;
		return(1);
		}; */

	numrows=t->img.ROWS;
	numcols=t->img.COLS;
	
	for (k=0; k<3; k++)
	     temp[k] = 0;
	DB_printf(("Entered ReadImagePPM PPM Ver %d\n", t->ver));

	NUM=(numrows)*(numcols);
	for(i=0;i<numrows;i++) {
	     for (j=0;j<numcols; j++) {
		  if(t->ver==0) {
		       for (k=0; k<3; k++) {
			    temp[k] = fgetc(fp1);
		       
			    if (ferror(fp1)==1) {
				 HIPL_ERRNO=-14;
				 clearerr(fp1);
				 return(-1);
			    }
		       
			    /* if (feof(fp1)==1) {
				 HIPL_ERRNO=-14;
				 clearerr(fp1);
				 return(-1);
				 } */
		       }

		       fputc(temp[0],t->fptr);
		       fputc(temp[1],t->jptr);
		       fputc(temp[2],t->kptr);
		       
		       if ((ferror(t->fptr)==1) || (ferror(t->jptr) == 1) ||
			   (ferror(t->kptr) == 1)) {
			    HIPL_ERRNO=-14;
			    clearerr(t->fptr);
			    return(-1);
		       }
		  }

		  if(t->ver==1) {
		       fscanf(fp1,"%d %d %d",&temp[0], &temp[1], &temp[2]);
		       if (ferror(fp1)==1) {
			    HIPL_ERRNO=-14;
			    clearerr(fp1);
			    return(-1);
		       }
		       /* if (feof(fp1)==1) {
			    HIPL_ERRNO=-14;
			    clearerr(fp1);
			    return(-1);
			    }*/
		       
		       fprintf(t->fptr,"%3d ",temp[0]);
		       fprintf(t->jptr,"%3d ",temp[1]);
		       fprintf(t->kptr,"%3d ",temp[2]);
		       
		       if ((ferror(t->fptr)==1) || (ferror(t->jptr) == 1) ||
			   (ferror(t->kptr) == 1)) {
			    HIPL_ERRNO=-14;
			    clearerr(t->fptr);
			    return(-1);
		       }
		  }
		  
	     }
	     
	}
	DB_printf(("Leaving ReadImagePPM\n"));

	return(0);
	
}

/********************************************************************
 *	FUNCTION: HIPL_WriteImagePPM
 *	PARAMETERS:
 *		FILE *fp         - File pointer for output file (I)
 *      THEADER t        - Header information (I)
 *	RETURN VALUES: 0 upon success
 *                 -1 upon failure
 *******************************************************************/
int HIPL_WriteImagePPM(FILE *fp, THEADER *t)
{
	int i,j;
	int numrows, numcols;
	FILE *id, *jd, *kd;
	int temp;
	long int NUM;
	NUM=(t->img.ROWS)*(t->img.COLS);

	/* if (HIPL_GetImageTypePGM(t) == 10) {
		if (HIPL_PutImageHeaderPGM(fp, t) != 0) {
			HIPL_ERRNO=-15;
			return(-1);
		}
	}
	if ((numrows = HIPL_GetImageRowsPGM(t)) <= 0) {
		HIPL_ERRNO=-16;
		return(-1);
	}
	if ((numcols = HIPL_GetImageColsPGM(t)) <= 0) {
		HIPL_ERRNO=-16;
		return(-1);
		} */

	if (t->ver == 0)
	     fprintf(fp, "P6\n%d %d\n%d\n", t->img.COLS, t->img.ROWS,
		     255);
	else
	     fprintf(fp, "P3\n%d %d\n%d\n", t->img.COLS, t->img.ROWS,
		     255);
	
	if(t->memflag!=1)
	{
	     if(ISUNIX) {
		  id=fopen(t->tempfname,"r");
		  jd=fopen(t->tempjname,"r");
		  kd=fopen(t->tempkname,"r");
		  
	     }		  
	     else
	     {    
		  if(t->ver==0)
		       id=fopen(t->tempfname,"rb");
		  else
		       id=fopen(t->tempfname,"r");
	     }
	     if((id==NULL) || (jd == NULL) || (kd == NULL))
	     {
		  HIPL_ERRNO=-7;
		  return(-1);
	     }
	     
	     for(i=0;i<NUM;i++){
		  if(t->ver==0)
		  {
		       temp=fgetc(id);
		       fputc(temp,fp);
		       temp = fgetc(jd);
		       fputc(temp, fp);
		       temp = fgetc(kd);
		       fputc(temp, fp);
		  }
		  if(t->ver==1)
		  {
		       fscanf(id,"%3d ",&temp);
		       fprintf(fp, "%3d ", temp);
		       fscanf(jd,"%3d ",&temp);
		       fprintf(fp, "%3d ", temp);
		       fscanf(kd,"%3d ",&temp);
		       fprintf(fp, "%3d ", temp);
		  }
	     }
	     fclose(id);
	     fclose(jd);
	     fclose(kd);
	}

	if(t->memflag==1)
	{
	     for(i=0;i<t->img.ROWS;i++)
	     {
		  for(j=0;j<t->img.COLS;j++)
		  {
		       if(t->ver==0) {
			    fputc(t->iarr[i][j],fp);
			    fputc(t->jarr[i][j],fp);
			    fputc(t->karr[i][j],fp);
		       }
		       if(t->ver==1) {
			    fprintf(fp,"%3d ",t->iarr[i][j]);
			    fprintf(fp,"%3d ",t->jarr[i][j]);
			    fprintf(fp,"%3d ",t->karr[i][j]);
			    if(ferror(fp)==1)
			    {
				 HIPL_ERRNO=-18;
				 clearerr(fp);
				 return(-1);
			    }
		       }
		  }
	     }
	}
	return(0);
}

