#include "incl.h"

extern int HIPL_Initialise(THEADER *T,char *name,short io,short arrtype,DATA *ID);
extern int HIPL_Closeall(THEADER *T);
extern int HIPL_AllocateMem(THEADER *T);
extern int HIPL_FreeMem(THEADER *);
extern int HIPL_Getpart(THEADER *T);
extern int HIPL_Putpart(THEADER *T);
extern int HIPL_Refresh(THEADER *T,short m);
extern void HIPL_Error(char *msg);
extern int HIPL_StoreResult(char *name,THEADER *T);
extern int HIPL_FindHistogram(THEADER *T,long int hist[257]);
int HIPL_RangeStretch(THEADER *,THEADER *,long int hist[257],int iter);

/****************************************************************
*	FUNCTION		: HIPL_RangeStretch
*	INPUT			: Input Filename,Output filename,histogram
*	OUTPUT			: 0 on success
*					  negative value on failure			
*   DESCRIPTION		: Stretches the input image each time in proportion
*                     to the maximum and minimum intensity values of 
*					  the image until it becomes a binary image. 	
*   EXTERNAL CALLS  : 1)HIPL_Getpart				
*			          2)HIPL_Putpart				
*					  3)HIPL_Error
*					  4)HIPL_Initialise	
*					  5)HIPL_StoreResult
*                     6)HIPL_Closeall
*					  7)HIPL_Refresh
*					  8)HIPL_FindHistogram		
*				
*   File			: 1)HIPL_Coreio.c
*					  2)hist.c					
*****************************************************************/

int HIPL_RangeStretch(THEADER *it,THEADER *output,long int hist[257],int iter)
{
	int Imin,Imax,Idiff,Omin,Omax,i,j,k,flag=0,iteration = 0,max;
	float Range,Fact,factor,val1,val2,new_val;
	DATA *ID;
	THEADER *input;
	input=(THEADER *)malloc(sizeof(THEADER));
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=it->img.ROWS;
	ID->COLS=it->img.COLS;
	ID->ver=it->ver;
	HIPL_Initialise(input,NULL,1,0,ID);
	HIPL_AllocateMem(input);
	do{
		max=HIPL_Getpart(it);
		for(i=0;i<max;i++)
			for(j=0;j<input->img.COLS;j++)
				input->iarr[i][j] = it->iarr[i][j];
			HIPL_Putpart(input);
	}while(input->P.nleft);
	
	HIPL_Refresh(input,0);
	hist=(long int*)malloc(257*sizeof(long int));
	i = HIPL_FindHistogram(input,hist);
	if (i<0)
	    {
		HIPL_Error("FindHistogram ");
		return(i);
	}
	// To find the maximum and minimum intensity values of the image 
	for(i=0;i<256;i++)
	{	
		if (hist[i] !=0)
		{
	        Imin = i;
		    break;
		}
	}
	for(j=256;j>0;j--)
	{
		if (hist[j] != 0)
		{
			Imax = j;
		    break;
		}
	}
	while(iteration<iter)
	{
		// If the input image range is within 0 and 255
		if(((Imin > 0)||(Imax < 255))&&(flag==0))
		{
			Idiff = Imax-Imin;
			Range =(float)0.1*Idiff;
			Fact=(float)Imin/(255-Imax+Imin);
			Omin =(int)(Imin - Fact*Range);
			Omax = (int)(Imax+Range*(1-Fact));
	
			if(Omin<0)
				Omin=0;
			if(Omax>255)
				Omax=255;
			factor = (float)(Omax-Omin)/(Imax-Imin);
			
			do
			{
				i=HIPL_Getpart(input);
				if(i<0)
				{
					HIPL_Error("Histogram Stretch");
					return(i);
				}
				for(j=0;j<i;j++)
				{
					for(k=0;k<input->img.COLS;k++)
					{
						val1 = (float)(input->iarr[j][k]-Imin);
						val2 = factor *val1;
						new_val = val2 + (float)Omin;
						output->iarr[j][k] = (int)new_val;
						if(output->iarr[j][k]>255)
							output->iarr[j][k]=255;
						if(output->iarr[j][k]<0)
							output->iarr[j][k]=0;
					}
				}

				i=HIPL_Putpart(output);
				if(i<0)
				{
					HIPL_Error("Histogram Stretch");
					return(i);
				}
				if(output->P.nleft==0)
					i=-1;
				if((Imin==0)&&(Imax==255))
					flag=1;
			}while(i>=0);
		}/*end of if*/
		//If the image range is at any of the bounds either 0 or 255
		else
			flag=1;
		if(flag==1)
		{
			Range=(float)0.05*(Imax-Imin);
			Omin = Imin+(int)Range;
			Omax = Imax-(int)Range;
			
			do
			{
				i=HIPL_Getpart(input);
				if(i<0)
				{
					HIPL_Error("Histogram Stretch");
					return(i);
				}
				for(j=0;j<i;j++)
				{
					for(k=0;k<input->img.COLS;k++)
					{
						if(input->iarr[j][k]<Omin)
							output->iarr[j][k]=0;
						if(input->iarr[j][k]>Omax)
							output->iarr[j][k]=255;

						factor=(float)255/(Omax-Omin);
						output->iarr[j][k]=(int)((input->iarr[j][k]-Omin)*factor);

						if(output->iarr[j][k]<0)
							output->iarr[j][k]=0;
						if(output->iarr[j][k]>255)
							output->iarr[j][k]=255;
					}
				}
					i=HIPL_Putpart(output);
				if(i<0)
				{
					HIPL_Error("Histogram Stretch");
					return(i);
				}
				if(output->P.nleft==0)
				i=-1;
			}while(i>=0);
		}
		HIPL_Refresh(input,0);
		HIPL_Refresh(output,0);
		do{
			max=HIPL_Getpart(output);
			for(i=0;i<max;i++)
				for(j=0;j<output->img.COLS;j++)
					input->iarr[i][j] = output->iarr[i][j];
			HIPL_Putpart(input);
		}while(input->P.nleft);
		HIPL_Refresh(input,0);
		HIPL_Refresh(output,0);
		iteration++;
		Imin=Omin;
		Imax=Omax;
	}
	HIPL_FreeMem(input);
	HIPL_Closeall(input);
	free(input);
	free(ID);
	return(0);
}
					
						
