#include"incl.h"

extern int HIPL_ERRNO;
extern int HIPL_Initialise(THEADER *it,char *fname,short io,short arrtype,DATA *ID);
extern int HIPL_AllocateMem(THEADER *it);
extern int HIPL_FreeMem(THEADER *it);
extern int HIPL_Getpart(THEADER *T);
extern int HIPL_Putpart(THEADER *T);
extern int HIPL_Refresh(THEADER *T,short i);
extern void HIPL_Error(char *msg);
extern int Create_temp_File(int b,THEADER *T);
extern int MAXMEM,MAXMEM1;

int HIPL_Newimg(THEADER *it,int rows,int cols,int grey);
int HIPL_Select(THEADER *T,THEADER *Sel,int xt,int yl,int xb,int yr,DATA *IDSel);
int HIPL_Dump(THEADER *it,THEADER *Sel,THEADER *ot,int xt,int yl,int xb,int yr);

/****************************************************************
*	FUNCTION : HIPL_Select					*
*								*
*								*
*	INPUT    : I/P THEADER ,SELECTION HEADER,		*
*								*
*	OUTPUT	 :      0 on success				*
*		       -1 on failure				*
*								*
*	DESCRIPTION : 1) fills in the header structure		*
*		      2) creates the temporary data files	*
*		      3) allocates memory to the array		*
*								*
****************************************************************/ 

int HIPL_Select(THEADER *T,THEADER *Sel,int xt,int yl,int xb,int yr,DATA *IDSel)
{

	int i,j,max;
	static int tcount=100;
	int part_size=0,num_cols=0,num_rows=0;
	int countc=0,countr=0;
	tcount++;
	i=Create_temp_File(tcount,Sel);
	if(i<0)
		return(-1);
	if(ISUNIX)
		Sel->fptr=fopen(Sel->tempfname,"w");
	else
	{
		if(Sel->ver==0)
			Sel->fptr=fopen(Sel->tempfname,"wb");
		else
			Sel->fptr=fopen(Sel->tempfname,"w");
	}
	if(Sel->fptr==NULL)
	{
		HIPL_ERRNO=-8;
		return(-1);
	}
	num_cols=yr-yl+1;
	num_rows=xb-xt+1;
	Sel->img.ROWS=num_rows;
	Sel->img.COLS=num_cols;
	IDSel->ROWS=num_rows;
	IDSel->COLS=num_cols;
	IDSel->ver=T->ver;
	do
	{
		max=HIPL_Getpart(T);
		if(max<0)
		{
			HIPL_Error("Getpart in Selection - input header");
			return(max);
		}
		if(T->ver==0)
		{
			for(i=0;i<max;i++)
			{
				countc=0;
				for(j=0;j<T->img.COLS;j++)
				{
					if((countr>=xt)&&(countr<=xb)&&(countc>=yl)&&(countc<=yr))
						fputc(T->iarr[i][j],Sel->fptr);
					countc++;
				}
				countr++;
			}	
		}
		if(T->ver==1)
		{
			for(i=0;i<max;i++)
			{
				countc=0;
				for(j=0;j<T->img.COLS;j++)
				{
					if((countr>=xt)&&(countr<=xb)&&(countc>=yl)&&(countc<=yr))
						fprintf(Sel->fptr,"%d ",T->iarr[i][j]);
					countc++;
				}
				countr++;
			}	
		}
		if(T->P.nleft==0)
				break;
		}while(max>0);
		fclose(Sel->fptr);
		Sel->P.psize=(int)((MAXMEM1*1024)/(Sel->img.COLS));
		if(Sel->P.psize > Sel->img.ROWS)
			Sel->P.psize=Sel->img.ROWS;
		part_size=Sel->P.psize;
		Sel->arrtype=0;
		Sel->P.rowsread=0;
		Sel->MROWS=0;
		Sel->MORGX=0;
		Sel->P.put=0;
		Sel->P.nleft=Sel->img.ROWS;
		Sel->ver=T->ver;
		Sel->iarr=(int **)malloc(part_size*sizeof(int *));
		if(Sel->iarr==(int **) 0)
		{
			HIPL_ERRNO=-5;
			return(-1);
		}
		for(i=0;i<part_size;i++)
		{
			Sel->iarr[i]=(int *)malloc(num_cols*sizeof(int));
			if(Sel->iarr[i]==(int *) 0)
			{
				HIPL_ERRNO=-5;
				return(-1);
			}
		}
	return(0);
}

/****************************************************************
*	FUNCTION : HIPL_Dump					*
*								*
*	INPUT    : I/P THEADER ,SELECTION THEADER,		*
*			O/P THEADER,				*
*			CO-ORDINATES				*
*								*
*	OUTPUT	 :      0 on success				*
*		       -1 on failure				*
*								*
*	DESCRIPTION : 1) Dumps the selected image into		*
*		        	output header structure		*
*								*
****************************************************************/ 

int HIPL_Dump(THEADER *it,THEADER *Sel,THEADER *ot,int xt,int yl,int xb,int yr)
{
int c=0,i,j,m=0,n=0,num_cols,num_rows,max1,max3,max2;
HIPL_Refresh(it,0);
HIPL_Refresh(ot,0);
HIPL_Refresh(Sel,0);
num_cols=yr-yl+1;
num_rows=xb-xt+1;
max1=HIPL_Getpart(Sel);
do {
	max2=HIPL_Getpart(it);
	for(i=0;i<max2;i++)
		{
		n=0;
		for(j=0;j<it->img.COLS;j++)
			{
			if(((c+i)>=xt)&&((c+i)<=xb)&&(j>=yl)&&(j<=yr)) {
			ot->iarr[i][j]=Sel->iarr[m][n];
			n++;
			if(n==num_cols) {
				m++;
				if(m==max1)
					{
					max1=HIPL_Getpart(Sel);
					m=0;
					}
				n=0;
				}
			}
			else
			ot->iarr[i][j]=it->iarr[i][j];
			}
		}
	c+=max2;
	max3=HIPL_Putpart(ot);
}while(ot->P.nleft);
return 0;
}			

/****************************************************************/ 

int HIPL_Newimg(THEADER *it,int rows,int cols,int grey)
{
	int i,j,left,max;
	DATA *ID;
	ID=(DATA *)malloc(sizeof(DATA));
	ID->ROWS=rows;
	ID->COLS=cols;
	ID->ver=0;
	HIPL_Initialise(it,NULL,1,0,ID);
	HIPL_AllocateMem(it);
	left = it->img.ROWS;
	do {
		if(left >= it->P.psize) max = it->P.psize;
		else max = left;
		for(i=0;i<max;i++)
			for(j=0;j<it->img.COLS;j++)
				it->iarr[i][j] = grey;
			left-=max;
		max = HIPL_Putpart(it);
	}while(it->P.nleft);
	free(ID);
	return 0;
}

